
// Linear algebra c functions


#include <stdio.h>
#include <math.h>
#define NMAX 100


void migs (int n, double a[NMAX][NMAX], double x[NMAX][NMAX])
/* Function to invert matrix a[][] with the inverse stored
   in x[][] in the output.  Copyright (c) Tao Pang 2001. */
{
  int i,j,k;
  double b[NMAX][NMAX];
  int indx[NMAX];
  void elgs();

  for (i = 0; i < n; ++i)
  {
    for (j = 0; j < n; ++j)
    {
      b[i][j] = 0;
    }
  }
  for (i = 0; i < n; ++i)
  {
    b[i][i] = 1;
  }

  elgs (n,a,indx);

  for (i = 0; i < n-1; ++i)
  {
    for (j = i+1; j < n; ++j)
    {
      for (k = 0; k < n; ++k)
      {
        b[indx[j]][k] = b[indx[j]][k]-a[indx[j]][i]*b[indx[i]][k];
      }
    }
  }

for (i = 0; i < n; ++i)
  {
    x[n-1][i] = b[indx[n-1]][i]/a[indx[n-1]][n-1];
    for (j = n-2; j >= 0; j = j-1)
    {
      x[j][i] = b[indx[j]][i];
      for (k = j+1; k < n; ++k)
      {
        x[j][i] = x[j][i]-a[indx[j]][k]*x[k][i];
      }
      x[j][i] = x[j][i]/a[indx[j]][j];
    }
  }
}

void elgs (int n , double a[NMAX][NMAX], int indx[NMAX])

/* Function to perform the partial-pivoting Gaussian elimination.
   a[][] is the original matrix in the input and transformed
   matrix plus the pivoting element ratios below the diagonal
   in the output.  indx[] records the pivoting order.
   Copyright (c) Tao Pang 2001. */
{
  int i, j, k=0, itmp;
  double c1, pi, pi1, pj;
  double c[n];

/* Initialize the index */

  for (i = 0; i < n; ++i)
  {
    indx[i] = i;
  }

/* Find the rescaling factors, one from each row */
 
  for (i = 0; i < n; ++i)
  {
    c1 = 0;
    for (j = 0; j < n; ++j)
    {
      if (fabs(a[i][j]) > c1) c1 = fabs(a[i][j]);
    }
    c[i] = c1;
  }

/* Search the pivoting (largest) element from each column */ 

  for (j = 0; j < n-1; ++j)
  {
    pi1 = 0;
    for (i = j; i < n; ++i)
    {
      pi = fabs(a[indx[i]][j])/c[indx[i]];
      if (pi > pi1)
      {
        pi1 = pi;
        k = i;
      }
    }

/* Interchange the rows via indx[] to record pivoting order */

    itmp = indx[j];
    indx[j] = indx[k];
    indx[k] = itmp;
    for (i = j+1; i < n; ++i)
    {
      pj = a[indx[i]][j]/a[indx[j]][j];

/* Record pivoting ratios below the diagonal */

      a[indx[i]][j] = pj;

/* Modify other elements accordingly */

      for (k = j+1; k < n; ++k)
      {
        a[indx[i]][k] = a[indx[i]][k]-pj*a[indx[j]][k];
      }
    }
  }
}
void matrix_copy(int n, int m, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)
    for(j=0;j<m;j++)
      Y[i][j] = X[i][j];
}

void matrix_copyld(int n, int m, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)
    for(j=0;j<m;j++)
      Y[i][j] = X[i][j];
}

void matrix_copypart(int n1, int n2, int m1, int m2, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{
  int i,j;
  for (i=n1;i<n2+1;i++)
    for(j=m1;j<m2+1;j++)
      Y[i-n1][j-m1] = X[i][j];
}

void matrix_copydtold(int n, int m, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)
    for(j=0;j<m;j++)
      Y[i][j] = (double) X[i][j];
}

void matrix_copyldtod(int n, int m, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)
    for(j=0;j<m;j++)
      Y[i][j] = (double) X[i][j];
}

void matrix_print(int n, int m, double X[NMAX][NMAX])
{
  int i,j;
    
  for (j=0;j<m;j++)
    for(i=0;i<n;i++) 
      printf("(%d,%d) matrix value is %15.15f\n",i,j,X[i][j]);
}

void bmp(int n, int m, double X[NMAX][NMAX], char *expression)
{
  puts(expression);
  matrix_print(n,m,X);
}

void mp(int n, int m, double X[n][m], char *expression)
{
  puts(expression);
  printmatrix(n,m,X);
}

void vp(int n, double *X, char *expression)
{
  puts(expression);
  printvec(n,X);
}

void vectorprint(int n, double X[n], char *expression)
{
  puts(expression);
  int i;
  for (i=0;i<n;i++)  
    printf("%f\n",X[i]);
} 

void ivp(int n, int *X, char *expression)
{
  puts(expression);
  printintvec(n,X);
}

void matrix_printld(int n, int m, double X[NMAX][NMAX])
{
  int i,j;
    
    for(i=0;i<n;i++) 
      for (j=0;j<m;j++)
	printf("(%d,%d) matrix value is %f\n",i,j,X[i][j]);
}

#define TOOSMALL ((double) 1.0e-9)
/*
	Cholesky decomposition can be done in-place.  Why I did not
	implement it that way:
		- It makes the code a little uglier,
		- Sometimes you want the original matrix to not be destroyed,
		- I'm not interested in matrices seriously larger
		than 100x100, which is 80k, so the wastage of storage
		is a total nonissue, especially in C.
*/

void cholesky(int N, double A[NMAX][NMAX], double U[NMAX][NMAX])
/*
Inputs: 
	N, integer
	A, N x N matrix _indexed_from_1_
Returns:
	U, N x N matrix, _indexed_from_1_, you must allocate before
		calling this routine,
	nullity, integer
*/
{
  int row, j, k,nullity=0;
  double sum;
  double tmpA[NMAX][NMAX],tmpU[NMAX][NMAX];
  int i;
  
  
  matrix_copyld(N,N,A,tmpA);

    for(i=0;i<N+1;i++) 
    for(j=0;j<N+1;j++)  
      A[i][j] = 0;

    for(i=0;i<N;i++) 
    for(j=0;j<N;j++)  
      A[i+1][j+1] = tmpA[i][j];
    //matrix_print(N+1,N+1,A);
  
	for (row=1; row<=N; row++) {
		/* First compute U[row][row] */
		sum = A[row][row];
		for (j=1; j<=(row-1); j++) sum -= U[j][row]*U[j][row];
		if (sum > TOOSMALL) {
			U[row][row] = sqrt(sum);
			/* Now find elements U[row][k], k > row. */
			for (k=(row+1); k<=N; k++) {
				sum = A[row][k];
				for (j=1; j<=(row-1); j++) 
					sum -= U[j][row]*U[j][k];
				U[row][k] = sum/U[row][row];
			}
		}
		else {	/* blast off the entire row. */
			for (k=row; k<=N; k++) U[row][k] = 0.0;
			(nullity)++;
		}
	}
	//matrix_print(N+1,N+1,U);
	matrix_copyld(N+1,N+1,U,tmpU);
	matrix_copyld(N+1,N+1,A,tmpA);

  for(i=0;i<N;i++)  {
    for(j=0;j<N;j++)  {
      U[i][j] = tmpU[i+1][j+1];    
      A[i][j] = tmpA[i+1][j+1];
    }
    }
	
}
/* End of cholesky.c */


void matrix_bigvectocolofbigmat(int n, int j, double y[NMAX], double Y[NMAX][NMAX])
{
  int i;
  for (i=0;i<n;i++)
    Y[i][j] = y[i];
}

void matrix_colofbigmattobigvec(int n, int j, double Y[NMAX][NMAX], double y[NMAX])
{
  int i;
  for (i=0;i<n;i++)
    y[i] = Y[i][j];
}

void matrix_colofbigmattoregvec(int n, int j, double Y[NMAX][NMAX], double *y)
{
  int i;
  for (i=0;i<n;i++)
    y[i] = Y[i][j];
}


void matrix_setpartofbigmatto0(int n, int m, double X[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)
    for(j=0;j<m;j++)
      X[i][j] = 0;
}

void matrix_setrestofbigmatto0(int n, int m, double X[NMAX][NMAX])
{
  int i,j;
  for (i=n;i<NMAX;i++)
    for(j=m;j<NMAX;j++)
      X[i][j] = 0;
}

void matrix_bigmattoregmat(int n, int m, double X[NMAX][NMAX], double Y[n][m])
{
  int i,j;
  for (i=0;i<n;i++) 
    for (j=0;j<m;j++)  
      Y[i][j] = X[i][j];
}

void matrix_regmattobigmat(int n, int m, double Y[n][m], double X[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++) 
    for (j=0;j<m;j++)  
      X[i][j] = Y[i][j];
}

void matrix_vecbigmattoregvec(int n, int m, double X[NMAX][NMAX], double Y[n*m])
{
  int i,j;
  //printf("I've gotten this far %d %d %d \n",n,m,n*m);  
  for (j=0;j<m;j++) {
    for (i=0;i<n;i++)  
      Y[i+j*n] = X[i][j];
  }
}

void bmrv(int n, int m, double X[NMAX][NMAX], double Y[n*m])
{
  int i,j;
  //printf("I've gotten this far %d %d %d \n",n,m,n*m);  
  for (j=0;j<m;j++) {
    for (i=0;i<n;i++)  
      Y[i+j*n] = X[i][j];
  }
}

void vec_bigvectoregvec(int n, double X[NMAX], double *Y)
{
  int i;
  for (i=0;i<n;i++) 
      Y[i] = X[i];
}

void vec_bigvectoregvec1(int n, double X[NMAX], double Y[n])
{
  int i;
  for (i=0;i<n;i++) 
      Y[i] = X[i];
}

void vec_regvectobigvec(int n, double *Y, double X[NMAX])
{
  int i;
  for (i=0;i<n;i++) 
      X[i] = Y[i];
}


void matrix_unvecregvectobigmat(int n, int m, double vecmatrix[n*m], double X[NMAX][NMAX])
{
  int i,j;

  for (j=0; j<m; j++) {
    for (i=0; i<n; i++) {
      X[i][j] = vecmatrix[i+j*n];
      //printf("This value %f\n",vecmatrix[i+j*n]);
    }
  }
}

void matrix_unvec(int n, int m, double *vecmatrix, double X[NMAX][NMAX])
{
  int i,j;

  for (j=0; j<m; j++) {
    for (i=0; i<n; i++) {
      X[i][j] = vecmatrix[i+j*n];
      //printf("This value %f\n",vecmatrix[i+j*n]);
    }
  }
}

void matrix_vec(int n, int m, double X[NMAX][NMAX], double *Y)
{
  int i,j;
  //printf("I've gotten this far %d %d %d \n",n,m,n*m);  
  for (j=0;j<m;j++) {
    for (i=0;i<n;i++)  
      Y[i+j*n] = X[i][j];
  }
}


void vec_setrestto0(int n, double y[NMAX])
{
  int i;
  for (i=n;i<NMAX;i++) 
    y[i]= 0;
}

void matrix_mult(int n, int m, int p, double A[NMAX][NMAX], double B[NMAX][NMAX], double C[NMAX][NMAX])
{
  int i,j,k;
  double tmp;
  for (i=0;i<n;i++)  {
    for (j=0;j<p;j++) { 
      for (k=0,tmp=0.0;k<m;k++) 
	tmp += A[i][k]*B[k][j];
      C[i][j] = tmp;
    }
  }
  
}

void matrix_multbyvec(int n, int m, double A[NMAX][NMAX], double b[NMAX], double out[NMAX])
{

  double B[NMAX][NMAX],C[NMAX][NMAX];
  
  matrix_bigvectocolofbigmat(m,0,b,B);
  matrix_mult(n,m,1,A,B,C);
  matrix_colofbigmattobigvec(n,0,C,out);

}

void rv_multbmbyrv(int n, int m, double A[NMAX][NMAX], double b[m], double out[n])
{
  int i,j;
  double tmp;
  for (i=0;i<n;i++)  {
    for(j=0,tmp=0;j<m;j++)
      tmp += A[i][j]*b[j];
    out[i] = tmp;
  }

}

void matrix_transpose(int n, int m, double A[NMAX][NMAX])
{
  int i,j;
  double B[NMAX][NMAX];

  for (i=0;i<n;i++)  {
    for (j=0;j<m;j++)  {
      B[j][i] = A[i][j];
    }
  }
  //bmp(m,n,B,"B");
  for (i=0;i<n+m;i++)  {
    for (j=0;j<n+m;j++)  {
      A[i][j] = B[i][j];
    }
  }
  //bmp(m,n,A,"A");
  //bmp(m+n,m+n,A,"A");

}

void matrix_transposetomat(int n, int m, double A[NMAX][NMAX], double B[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)  {
    for (j=0;j<m;j++)  {
      B[j][i] = A[i][j];
    }
  }
}

void matrix_crossprod(int n, int m, double A[NMAX][NMAX], double B[NMAX][NMAX])
{
  double C[NMAX][NMAX];
  matrix_transposetomat(n,m,A,C);
  //printf("Printing transpose\n"); 
  //printpartofmatrix(NMAX,NMAX,m,n,C);
  //return;
  matrix_mult(m,n,m,C,A,B);

}


void matrix_scale(int n, int m, double scalar, double A[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)  
    for (j=0;j<m;j++)  
      A[i][j] = A[i][j]*scalar;

}


void matrix_addscale(int n, int m , double scalar1, double scalar2, double A[NMAX][NMAX], double B[NMAX][NMAX], 
			 double C[NMAX][NMAX])
{
  int i,j;
  for (i=0;i<n;i++)  
    for (j=0;j<m;j++)  
      C[i][j] = A[i][j]*scalar1 + B[i][j]*scalar2;

}

void vec_addscale(int n, double scalar1, double scalar2, double x[NMAX], double y[NMAX], 
		     double z[NMAX])
{
  int i;
  for (i=0;i<n;i++)  
    z[i] = x[i]*scalar1 + y[i]*scalar2;

}


void matrix_cholesky(int n, double A[NMAX][NMAX], double U[NMAX][NMAX])
{ 
  // A = U%*%U^T, return U
  int i, j, k;

  matrix_copyld(n,n,A,U);
  //printf("In cholesky A\n");
  //matrix_printld(n,n,A);
  //printf("This is U\n");
  //printpartofmatrix(NMAX,NMAX,n,n,U);

  for(i=0;i<n-1;i++)  {
    U[i][i] = sqrt(U[i][i]);
    for (j=i+1;j<n;j++)  
      U[j][i] = U[j][i]/U[i][i];
    for (j=i+1;j<n;j++)  {
      for(k=j;k<n;k++)  
	U[k][j] -= U[j][i]*U[k][i];
    }
  }
  U[n-1][n-1] = sqrt(U[n-1][n-1]);
  for(i=0;i<n-1;i++)  
    for (j=i+1;j<n;j++)  
      U[i][j] = 0;

  //printf("In cholesky U\n");
  //matrix_printld(n,n,U);
    
}
/* End of cholesky.c */


void matrix_inverseLT(int n, double A[NMAX][NMAX], double UU[NMAX][NMAX])
{
  int i, j, k;
  double tmp;
  //printf("I am about to print\n");
  //printpartofmatrix(NMAX,NMAX,n,n,A);
  //return;
  //printpartofmatrix(NMAX,NMAX,n,n,A);
  //return;

  matrix_setpartofbigmatto0(n,n,UU);
  for(i=0;i<n;i++)
    UU[i][i] = 1.0/A[i][i];
  for(i=0;i<n-1;i++)  {
    for(j=i+1;j<n;j++)  {
      for(k=0,tmp=0;k<j;k++) 
	tmp += A[j][k]*UU[k][i];
      UU[j][i]= -tmp/A[j][j];
    }
  }
  //printf("Printing matrixinverUU\n");
  //matrix_print(n,n,UU);
  //printf("Printing matrixinverLT\n");
  //printpartofmatrix(NMAX,NMAX,n,n,U);
    
}

void matrix_UTsolve(int n, int p, double A[NMAX][NMAX], double m[NMAX], double b[NMAX])
{
  // A(n,p), m(n), b(p) where A is upper triangular with nonzero entries on the diagonal
  // solves the system Ab=m for b
  int j, k;
  double tmp;

  //printf("I am about to print\n");
  //printpartofmatrix(NMAX,NMAX,n,n,A);
  //return;
  //printpartofmatrix(NMAX,NMAX,n,n,A);
  //return;

  b[p-1] = m[p-1]/A[p-1][p-1];

  for(j=p-2;j>-1;j--)  {
    for(k=p-1,tmp=0;k>j;k--)
      tmp += A[j][k]*b[k];
    b[j] = (m[j] - tmp)/A[j][j];
  }

    
}

void matrix_inversePD(int n, double A[NMAX][NMAX], double U[NMAX][NMAX])
{
  double A_chol[NMAX][NMAX],A_chol_inv[NMAX][NMAX];

  //printf("I am herePD\n");
  //printpartofmatrix(NMAX,NMAX,n,n,A);
  matrix_cholesky(n,A,A_chol);
  //printf("printintg A_chol\n"); 
  //matrix_print(n,n,A_chol);
  //double tmpmat1[NMAX][NMAX],tmpmat2[NMAX][NMAX],tmpmat3[NMAX][NMAX];
  //matrix_copy(n,n,A_chol,tmpmat1);
  //matrix_transpose(n,n,tmpmat1);
  //matrix_crossprod(n,n,tmpmat1,tmpmat3);
  //matrix_addscale(n,n,1.0,-1.0,tmpmat3,A,tmpmat2);
  //printf("Did cholesky work\n");
  //matrix_print(n,n,tmpmat2);
  matrix_inverseLT(n,A_chol,A_chol_inv);
  //printf("Printing A_chol_inv\n");
  //matrix_print(n,n,A_chol_inv); 
  //printf("Printinf crossprod\n");
  //return;
  matrix_crossprod(n,n,A_chol_inv,U);
  //double tmpmat1[NMAX][NMAX],tmpmat2[NMAX][NMAX],tmpmat3[NMAX][NMAX];
  //matrix_mult(n,n,n,A,U,tmpmat1);
  //matrix_mult(n,n,n,U,A,tmpmat2);
  //matrix_addscale(n,n,1.0,-1.0,tmpmat1,tmpmat2,tmpmat3);
  //printf("Did inverse work\n");
  //matrix_print(n,n,tmpmat3);
  //return;
  //printpartofmatrix(NMAX,NMAX,n,n,U);
}
    
void matrix_lm(int n, int p, double y[NMAX], double X[NMAX][NMAX], double beta[NMAX])
{  
  double tX[NMAX][NMAX],tXX[NMAX][NMAX];
  
  matrix_crossprod(n,p,X,tXX);
  matrix_transposetomat(n,p,X,tX);

  double chol[NMAX][NMAX],m1[NMAX],m2[NMAX],A_chol_inv[NMAX][NMAX];
  matrix_cholesky(p,tXX,chol);  
  //printf("printing chol of tXX in lm\n"); 
  //matrix_print(p,p,chol);  
  matrix_multbyvec(p,n,tX,y,m1);
  //printf("printing tXy\n"); 
  //printvec(p,m1);  
  matrix_inverseLT(p,chol,A_chol_inv);
  //printf("printing inv of chol\n"); 
  //matrix_print(p,p,A_chol_inv);  
  matrix_multbyvec(p,p,A_chol_inv,m1,m2);
  //printf("printing m2\n"); 
  //printvec(p,m2);  
  matrix_transpose(p,p,chol);
  matrix_UTsolve(p,p,chol,m2,beta); 
  //printf("printing alternative beta\n");
  //printvec(p,b);
}
    
void matrix_proj_nd(int n, int p, double X[NMAX][NMAX], double P[NMAX][NMAX])
{  
  double tX[NMAX][NMAX],tXX[NMAX][NMAX],tXXinv[NMAX][NMAX],XtXXinv[NMAX][NMAX];

  matrix_transposetomat(n,p,X,tX);
  matrix_mult(p,n,p,tX,X,tXX);
  //printf("printing tXX\n");
  //matrix_print(p,p,tXX);
  //matrix_scale(p,p,-1.0,tXX);
  matrix_inversePD(p,tXX,tXXinv);
  //printf("printing tXXinv\n");
  //matrix_print(p,p,tXXinv);
  matrix_mult(n,p,p,X,tXXinv,XtXXinv);
  //printf("printing X\n");
  //matrix_print(n,p,X);
  //printf("printing XtXXinv\n");
  //matrix_print(n,p,XtXXinv);
  matrix_mult(n,p,n,XtXXinv,tX,P);

}

// kronecker calculates the kronecker product of the matrices A and B
// A a m by n matrix
// B a p by q matrix 
// output, a mp by nq matrix krocker(A,B) 

void kronecker(int m, int n, int p, int q, double A[NMAX][NMAX], double B[NMAX][NMAX], double Y[NMAX][NMAX])
{  
  int i,j,k,l;
 
  for(i=0;i<m;i++)  {
    for(j=0;j<n;j++)  {
      for (k=0;k<p;k++)  {  
	for (l=0;l<q;l++)  
	  Y[i*p+k][j*q+l] = A[i][j]*B[k][l];
      }
    }
  }
}

// ss calculates the sums of squares of vector
// x a n length vector
// output, a scalar sum_{i=1}^n x[i]^2

double ss(int n, double *x)
{  
  int i;
  double out;
  
  for(i=0,out=0.0;i<n;i++)  
    out += pow(x[i],2.0);
  return out;
}


// ginvgrenville calculates the generalized inverse (using Grevilles algorithm) of a n by m+1 partitioned matrix [A_m,a_{m+1}] 
// A a n by m+1 matrix
// output, a m+1 by n matrix Aginv, s.t A%*%Aginv%*%A=A.

void ginvgrenville(int n, int m, double A[NMAX][NMAX], double Aginv[NMAX][NMAX])
{  
  int i,j;
  double d[m],b[n],c[n],B[NMAX][NMAX];
  double tmp,ssc,ssd,eps=1.0E-20;
  matrix_copy(m,n,Aginv,B);

  for (j=0;j<m;j++)  {
    for (i=0,tmp=0.0;i<n;i++)  
      tmp += B[j][i]*A[i][m];
    d[j] = tmp;
  }
  //vp(m,d,"d");
  for (i=0;i<n;i++)  {
    for (j=0,tmp=0.0;j<m;j++)  
      tmp += A[i][j]*d[j];
    c[i] = A[i][m]-tmp;
  }
  //vp(n,c,"c");
  ssc = ss(n,c);
  if (ssc > eps)  {
    for (i=0;i<n;i++) 
      b[i] = c[i]/ssc;
  } else {
    ssd = ss(m,d);
    for (i=0;i<n;i++)  {
      for (j=0,tmp=0.0;j<m;j++)  
	tmp += d[j]*B[j][i];   
      b[i] = tmp/(1.0+ssd);
    }
  }
  //vp(n,b,"b");
  for (j=0;j<m;j++)  {
    for (i=0;i<n;i++)  
      Aginv[j][i] = B[j][i]-d[j]*b[i];
  }
  for (i=0;i<n;i++)  
    Aginv[m][i] = b[i];
  
  //bmp(n,m+1,A,"AAA");
  //bmp(m+1,n,Aginv,"BBB");

}

void ginv(int n, int m, double A[NMAX][NMAX], double B[NMAX][NMAX])
{  
  int i,j;
  double a1[n],ssa1,eps=1.0E-20;
  
  //printf("n = %d,m = %d\n",n,m);
  for (i=0;i<n;i++)  {
    a1[i] = A[i][0];
  //  printf("a1[%d]=%f,A[%d]=%f\n",i,a1[i],i,A[i][0]);
  }
  //vectorprint(n,a1,"a1");
  ssa1 = ss(n,a1);
  if ( ssa1 < eps)  {
    for (i=0;i<n;i++)  
      B[0][i] = 0.0;
  } else  {
    for (i=0;i<n;i++)  
      B[0][i] = A[i][0]/ssa1;
  }
  //printf("ss %f\n",ssa1);
  //bmp(n,m,A,"A");
  //bmp(1,n,B,"B");
  for (j=1;j<m;j++)  {  
    ginvgrenville(n,j,A,B);
    //bmp(m,j+1,B,"Bi");
  }
  //bmp(m,n,B,"B");
}

void ginvwrapper(int *n, int *m, double *A, double *B)
{  
  double A_m[NMAX][NMAX],B_m[NMAX][NMAX];
  
  //vp(*n**m,A,"A");
  matrix_unvec(*n,*m,A,A_m);
 //bmp(*n,*m,A_m,"A");
  ginv(*n,*m,A_m,B_m);
 //bmp(*m,*n,B_m,"M");
  matrix_vec(*m,*n,B_m,B);
}

void kroneckerwrapper(int *m, int *n, int *p, int *q, double *A, double *B, double *out)
{  
  double A_m[NMAX][NMAX],B_m[NMAX][NMAX],out_m[NMAX][NMAX];
  
  //vp(*n**m,A,"A");
  matrix_unvec(*m,*n,A,A_m);
  matrix_unvec(*p,*q,B,B_m);
  //bmp(*n,*m,A_m,"A");
  kronecker(*m,*n,*p,*q,A_m,B_m,out_m);
  matrix_vec(*m**p,*n**q,out_m,out);
}
  
// commutmat calculates the commutation matrix nm by bm matrix K, i.e. if A is an m x n matrix then Kmn%*%vec(A) = vec(t(A)) 
// output, a mn by mn matrix 
void commutmat(int m, int n, double out[NMAX][NMAX])
{  
  int i,j,k,l;
  double H[NMAX][NMAX],tH[NMAX][NMAX],tmp[NMAX][NMAX];
 
  matrix_setpartofbigmatto0(m*n,m*n,out);
  for (i=0;i<m;i++)  {
    for (j=0;j<n;j++)  {
      matrix_setpartofbigmatto0(m,n,H);
      matrix_setpartofbigmatto0(n,m,tH);
      H[i][j] = 1.0;
      tH[j][i] = 1.0;
      kronecker(m,n,n,m,H,tH,tmp);
      for (k=0;k<m*n;k++)  {
	for (l=0;l<m*n;l++)  
	  out[k][l] += tmp[k][l];
      }
    }
  }
}

void commutmatwrapper(int *m, int *n, double *X)
{  
  double X_m[NMAX][NMAX];
  
  commutmat(*m,*n,X_m);
  matrix_vec(*m**n,*m**n,X_m,X);
}


// mpgrad calculates the gradient of the moore penrose inverse of X 
// output, a nm by nm matrix Y = d vec(Xmp)/ d vec(X)^T
// see Schott for formula  
void Xmpgrad(int m, int n, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{  
  int i,j;
  double Xmp[NMAX][NMAX],tXmp[NMAX][NMAX],tX[NMAX][NMAX],tXmpXmp[NMAX][NMAX],XmpX[NMAX][NMAX],XXmp[NMAX][NMAX],XmptXmp[NMAX][NMAX],Kmn[NMAX][NMAX];
  double InminusXmpX[NMAX][NMAX],ImminusXXmp[NMAX][NMAX],mat1[NMAX][NMAX],mat2[NMAX][NMAX],mat3[NMAX][NMAX],mat4[NMAX][NMAX],mat5[NMAX][NMAX];
  ginv(m,n,X,Xmp);
  //bmp(n,m,Xmp,"Xmp");
  matrix_transposetomat(m,n,X,tX);
  matrix_transposetomat(n,m,Xmp,tXmp);
  matrix_mult(m,n,m,tXmp,Xmp,tXmpXmp);
  matrix_mult(n,m,n,Xmp,X,XmpX);
  //bmp(n,n,XmpX,"XmpX");
  //bmp(m,n,X,"dX");
  matrix_mult(m,n,m,X,Xmp,XXmp);
  matrix_mult(n,m,n,Xmp,tXmp,XmptXmp);
  commutmat(m,n,Kmn);
  // note that XmpX and XXmp are mislabled
  for (i=0;i<n;i++)  {
    for (j=0;j<n;j++)  {
      if (i==j)  {
	InminusXmpX[i][j] = 1.0-XmpX[i][j];
      } else {
	InminusXmpX[i][j] = -XmpX[i][j];
      }
    }
  }
  for (i=0;i<m;i++)  {
    for (j=0;j<m;j++)  {
      if (i==j)  {
	ImminusXXmp[i][j] = 1.0-XXmp[i][j];
      } else {
	ImminusXXmp[i][j] = -XXmp[i][j];
      }
    }
  }
  kronecker(m,m,n,n,tXmpXmp,InminusXmpX,mat1);
  kronecker(m,m,n,n,ImminusXXmp,XmptXmp,mat2);
  kronecker(m,n,n,m,tXmp,Xmp,mat3);
  matrix_addscale(m*n,m*n,1.0,1.0,mat1,mat2,mat4);
  matrix_mult(m*n,m*n,m*n,mat4,Kmn,mat5);
  //bmp(m,m,tXmpXmp,"tXmpXmp");
  //bmp(n,n,InminusXmpX,"InminusXmpX");
  //bmp(m*n,m*n,mat1,"mat1");
  //bmp(m*n,m*n,mat2,"mat2");
  //bmp(m*n,m*n,mat3,"mat3");
  //bmp(m*n,m*n,mat4,"mat4");
  //bmp(m*n,m*n,mat5,"mat5");
  matrix_addscale(m*n,m*n,1.0,-1.0,mat5,mat3,Y);
  //bmp(m*n,m*n,Y,"Y");
}


void Xmpgradwrapper(int *m, int *n, double *X, double *Y)
{  
  double X_m[NMAX][NMAX],Y_m[NMAX][NMAX];
  
  vp(*m**n,X,"Xvec");
  matrix_unvec(*m,*n,X,X_m);
  Xmpgrad(*m,*n,X_m,Y_m);
  matrix_vec(*m**n,*m**n,Y_m,Y);
}

// ident te identity matrix
// output, a n by n matrix 
void identity(int n, double out[NMAX][NMAX])
{  
  int i,j;
 
  for (i=0;i<n;i++) {
    for (j=0;j<n;j++)  
      out[i][j] = (i==j) ? 1.0 : 0.0;
  }

}

// XmptXmp calculates the gradient of t(Xmp)%*%Xmp 
// output, a mm by nm matrix Y = d vec(t(Xmp)%*%Xmp)/ d vec(X)^T
// kronecker(t(Xmp),diag(m))%*%commutmat(n,m)+kronecker(diag(m),t(Xmp)))%*%Xmp.grad(X)
void XmptXmpgrad(int m, int n, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{  

  double Xmp[NMAX][NMAX],Xmpt[NMAX][NMAX],dXmp[NMAX][NMAX],Im[NMAX][NMAX],kronXmptIm[NMAX][NMAX],kronImXmpt[NMAX][NMAX],kronXmptImKnm[NMAX][NMAX];
  double kronXmptImKnmpluskronImXmpt[NMAX][NMAX],Knm[NMAX][NMAX];

  ginv(m,n,X,Xmp);
  matrix_transposetomat(n,m,Xmp,Xmpt);
  Xmpgrad(m,n,X,dXmp);
  commutmat(n,m,Knm);
  identity(m,Im);
  kronecker(m,n,m,m,Xmpt,Im,kronXmptIm);
  kronecker(m,m,m,n,Im,Xmpt,kronImXmpt);
  matrix_mult(m*m,m*n,m*n,kronXmptIm,Knm,kronXmptImKnm);
  matrix_addscale(m*m,m*n,1.0,1.0,kronXmptImKnm,kronImXmpt,kronXmptImKnmpluskronImXmpt);
  matrix_mult(m*m,m*n,m*n,kronXmptImKnmpluskronImXmpt,dXmp,Y);
}

// XtXmptgrad calculates the gradient of t(X)%*%t(Xmp) 
// output, a nn by nm matrix Y = d vec(t(X)%*%t(Xmp))/ d vec(X)^T
// kronecker((Xmp),diag(n))%*%commutmat(m,n)+kronecker(diag(n),t(X))%*%commutmat(n,m)%*%Xmp.grad(X)
void XtXmptgrad(int m, int n, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{  

  double Xmp[NMAX][NMAX],Xt[NMAX][NMAX],dXmp[NMAX][NMAX],Kmn[NMAX][NMAX],Knm[NMAX][NMAX],In[NMAX][NMAX],kronXmpIn[NMAX][NMAX];
  double kronInXt[NMAX][NMAX],kronInXtKnm[NMAX][NMAX],kronXmpInKmn[NMAX][NMAX],kronInXtKnmdXmp[NMAX][NMAX];
  ginv(m,n,X,Xmp);
  matrix_transposetomat(m,n,X,Xt);
  Xmpgrad(m,n,X,dXmp);
  commutmat(m,n,Kmn);
  commutmat(n,m,Knm);
  identity(n,In);
  kronecker(n,m,n,n,Xmp,In,kronXmpIn);
  kronecker(n,n,n,m,In,Xt,kronInXt);
  matrix_mult(n*n,m*n,m*n,kronXmpIn,Kmn,kronXmpInKmn);
  matrix_mult(n*n,m*n,m*n,kronInXt,Knm,kronInXtKnm);
  matrix_mult(n*n,m*n,m*n,kronInXtKnm,dXmp,kronInXtKnmdXmp);
  matrix_addscale(n*n,m*n,1.0,1.0,kronXmpInKmn,kronInXtKnmdXmp,Y);
}

// XmptXtgrad calculates the gradient of t(Xmp)%*%t(X) 
// output, a mm by nm matrix Y = d vec(t(Xmp)%*%t(X)) / d vec(X)^T
// kronecker((X),diag(m))%*%commutmat(n,m)%*%Xmp.grad(X)+kronecker(diag(m),t(Xmp))%*%commutmat(m,n)
void XmptXtgrad(int m, int n, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{  
  double Xmp[NMAX][NMAX],Xmpt[NMAX][NMAX],dXmp[NMAX][NMAX],kronXIm[NMAX][NMAX],kronImXmpt[NMAX][NMAX],kronXImKnm[NMAX][NMAX];
  double kronXImKnmdXmp[NMAX][NMAX],kronImXmptKmn[NMAX][NMAX],Knm[NMAX][NMAX],Kmn[NMAX][NMAX],Im[NMAX][NMAX];
  ginv(m,n,X,Xmp);
  matrix_transposetomat(n,m,Xmp,Xmpt);
  Xmpgrad(m,n,X,dXmp);
  commutmat(n,m,Knm);
  commutmat(m,n,Kmn);
  identity(m,Im);
  kronecker(m,n,m,m,X,Im,kronXIm);
  kronecker(m,m,m,n,Im,Xmpt,kronImXmpt);
  matrix_mult(m*m,m*n,m*n,kronXIm,Knm,kronXImKnm);
  matrix_mult(m*m,m*n,m*n,kronXImKnm,dXmp,kronXImKnmdXmp);
  matrix_mult(m*m,m*n,m*n,kronImXmpt,Kmn,kronImXmptKmn);
  matrix_addscale(m*m,m*n,1.0,1.0,kronXImKnmdXmp,kronImXmptKmn,Y);
}

// XmpXmptgrad calculates the gradient of the matrix Xmp%*%t(Xmp) 
// output, a nn by nm matrix Y = d vec((Xmp)%*%t(Xmp))/ d vec(X)^T
// (kronecker((Xmp),diag(n))+kronecker(diag(n),(Xmp))%*%commutmat(n,m))%*%Xmp.grad(X)
void XmpXmptgrad(int m, int n, double X[NMAX][NMAX], double Y[NMAX][NMAX])
{  

  double Xmp[NMAX][NMAX],dXmp[NMAX][NMAX],Knm[NMAX][NMAX],kronXmpIn[NMAX][NMAX],kronInXmpKnm[NMAX][NMAX];
  double In[NMAX][NMAX],kronInXmp[NMAX][NMAX],kronXmpInpluskronInXmpKnm[NMAX][NMAX];
  ginv(m,n,X,Xmp);
  Xmpgrad(m,n,X,dXmp);
  commutmat(n,m,Knm);
  identity(n,In);
  kronecker(n,m,n,n,Xmp,In,kronXmpIn);
  kronecker(n,n,n,m,In,Xmp,kronInXmp);
  matrix_mult(n*n,m*n,m*n,kronInXmp,Knm,kronInXmpKnm);
  matrix_addscale(n*n,m*n,1.0,1.0,kronXmpIn,kronInXmpKnm,kronXmpInpluskronInXmpKnm);
  matrix_mult(n*n,m*n,m*n,kronXmpInpluskronInXmpKnm,dXmp,Y);
}

void XmptXmpgradwrapper(int *m, int *n, double *X, double *Y)
{  
  double X_m[NMAX][NMAX],Y_m[NMAX][NMAX];
  
  matrix_unvec(*m,*n,X,X_m);
  XmptXmpgrad(*m,*n,X_m,Y_m);
  matrix_vec(*m**m,*m**n,Y_m,Y);
}
void XtXmptgradwrapper(int *m, int *n, double *X, double *Y)
{  
  double X_m[NMAX][NMAX],Y_m[NMAX][NMAX];
  
  matrix_unvec(*m,*n,X,X_m);
  XtXmptgrad(*m,*n,X_m,Y_m);
  matrix_vec(*n**n,*m**n,Y_m,Y);
}
void XmptXtgradwrapper(int *m, int *n, double *X, double *Y)
{  
  double X_m[NMAX][NMAX],Y_m[NMAX][NMAX];
  
  matrix_unvec(*m,*n,X,X_m);
  XmptXtgrad(*m,*n,X_m,Y_m);
  matrix_vec(*m**m,*m**n,Y_m,Y);
}
void XmpXmptgradwrapper(int *m, int *n, double *X, double *Y)
{  
  double X_m[NMAX][NMAX],Y_m[NMAX][NMAX];
  
  matrix_unvec(*m,*n,X,X_m);
  XmpXmptgrad(*m,*n,X_m,Y_m);
  matrix_vec(*n**n,*m**n,Y_m,Y);
}

void print3darray(int m, int n, int o, double X[m][n][o])
{  
  int i,j,k;

  for (i=0;i<m;i++)  {
    for (k=0;k<o;k++)  {
      for (j=0;j<n;j++) 
	printf("(%d,%d,%d) matrix value is %f\n",i,j,k,X[i][j][k]);  
      }
  }
}

// Xmphess calculates the hessian of the matrix (Xmp) 
// output, mn by mn by mn array where the first compenent contains  vec((X)) and second and third elements contain d vec(X) d vec(X)^T
// 
void Xmphess(int m, int n, double X[NMAX][NMAX], double d2Xmp[m*n][m*n][m*n])
{  
  int i,j,ii,jj,k,l;
  double Xmp[NMAX][NMAX],dXmp[NMAX][NMAX],Knm[NMAX][NMAX],kronXmpIn[NMAX][NMAX],kronInXmpKnm[NMAX][NMAX];
  double In[NMAX][NMAX],kronInXmp[NMAX][NMAX],kronXmpInpluskronInXmpKnm[NMAX][NMAX];
  double Xmpt[NMAX][NMAX],kronImXmpt[NMAX][NMAX],Im[NMAX][NMAX],kronXmptImKnm[NMAX][NMAX];
  double kronXmptIm[NMAX][NMAX],kronXmptImKnmpluskronImXmpt[NMAX][NMAX],dXmptXmp[NMAX][NMAX],Xt[NMAX][NMAX],Kmn[NMAX][NMAX];
  double kronInXt[NMAX][NMAX],kronInXtKnm[NMAX][NMAX],kronInXtKnmdXmp[NMAX][NMAX],kronXIm[NMAX][NMAX],kronXImKnm[NMAX][NMAX];
  double kronXmpInKmn[NMAX][NMAX],dXtXmpt[NMAX][NMAX],kronXImKnmdXmp[NMAX][NMAX],kronImXmptKmn[NMAX][NMAX],dXmptXt[NMAX][NMAX],dXmpXmpt[NMAX][NMAX];
  double XtXmpt[NMAX][NMAX],InminusXtXmpt[NMAX][NMAX],XmptXmp[NMAX][NMAX],XmpXmpt[NMAX][NMAX],XmptXt[NMAX][NMAX],ut[NMAX][NMAX],vt[NMAX][NMAX];
  double ImminusXmptXt[NMAX][NMAX],InminusXtXmptjj[NMAX][NMAX],XmpXmptjj[NMAX][NMAX],XmptXmpii[NMAX][NMAX],ImminusXmptXtii[NMAX][NMAX];
  double Xmpii[NMAX][NMAX],kronutIm[NMAX][NMAX],kronvtIn[NMAX][NMAX],kronutIn[NMAX][NMAX],kronImvt[NMAX][NMAX];
  double Xmptjj[NMAX][NMAX],kronvtIndXtXmpt[NMAX][NMAX],kronutImdXmptXt[NMAX][NMAX],kronvtIndXmpXmpt[NMAX][NMAX];
  double mat1[NMAX][NMAX],mat2[NMAX][NMAX],mat3[NMAX][NMAX],mat4[NMAX][NMAX],mat5[NMAX][NMAX],mat6[NMAX][NMAX];
  double kronutIndXmp[NMAX][NMAX],kronImvtdXmp[NMAX][NMAX],kronutImdXmptXmp[NMAX][NMAX];
  
  ginv(m,n,X,Xmp);
  matrix_transposetomat(n,m,Xmp,Xmpt);
  Xmpgrad(m,n,X,dXmp);
  commutmat(m,n,Kmn);
  commutmat(n,m,Knm);
  identity(m,Im);
  kronecker(m,n,m,m,Xmpt,Im,kronXmptIm);
  kronecker(m,m,m,n,Im,Xmpt,kronImXmpt);
  matrix_mult(m*m,m*n,m*n,kronXmptIm,Knm,kronXmptImKnm);
  matrix_addscale(m*m,m*n,1.0,1.0,kronXmptImKnm,kronImXmpt,kronXmptImKnmpluskronImXmpt);
  //return;
  //bmp(m*m,m*n,kronXmptImKnm,"kronXmptImKnm");
  //bmp(m*m,m*n,kronImXmpt,"kronImXmpt");
  //bmp(m*m,m*n,kronXmptImKnmpluskronImXmpt,"kronXmptImKnmpluskronImXmpt");
  //bmp(m*n,m*n,dXmp,"dXmp");
  matrix_mult(m*m,m*n,m*n,kronXmptImKnmpluskronImXmpt,dXmp,dXmptXmp);
  //bmp(m*m,m*n,dXmptXmp,"dXmptXmp");
  matrix_transposetomat(m,n,X,Xt);
  //return;
  identity(n,In);
  kronecker(n,m,n,n,Xmp,In,kronXmpIn);
  kronecker(n,n,n,m,In,Xt,kronInXt);
  matrix_mult(n*n,m*n,m*n,kronXmpIn,Kmn,kronXmpInKmn);
  matrix_mult(n*n,m*n,m*n,kronInXt,Knm,kronInXtKnm);
  matrix_mult(n*n,m*n,m*n,kronInXtKnm,dXmp,kronInXtKnmdXmp);
  matrix_addscale(n*n,m*n,1.0,1.0,kronXmpInKmn,kronInXtKnmdXmp,dXtXmpt);
  kronecker(m,n,m,m,X,Im,kronXIm);
  kronecker(m,m,m,n,Im,Xmpt,kronImXmpt);
  matrix_mult(m*m,m*n,m*n,kronXIm,Knm,kronXImKnm);
  matrix_mult(m*m,m*n,m*n,kronXImKnm,dXmp,kronXImKnmdXmp);
  matrix_mult(m*m,m*n,m*n,kronImXmpt,Kmn,kronImXmptKmn);
  matrix_addscale(m*m,m*n,1.0,1.0,kronXImKnmdXmp,kronImXmptKmn,dXmptXt);
  kronecker(n,n,n,m,In,Xmp,kronInXmp);
  matrix_mult(n*n,m*n,m*n,kronInXmp,Knm,kronInXmpKnm);
  matrix_addscale(n*n,m*n,1.0,1.0,kronXmpIn,kronInXmpKnm,kronXmpInpluskronInXmpKnm);
  matrix_mult(n*n,m*n,m*n,kronXmpInpluskronInXmpKnm,dXmp,dXmpXmpt);
  matrix_mult(n,m,n,Xt,Xmpt,XtXmpt);
  matrix_addscale(n,n,1.0,-1.0,In,XtXmpt,InminusXtXmpt);
  matrix_mult(m,n,m,Xmpt,Xmp,XmptXmp);
  matrix_mult(n,m,n,Xmp,Xmpt,XmpXmpt);
  matrix_mult(m,n,m,Xmpt,Xt,XmptXt);
  matrix_addscale(m,m,1.0,-1.0,Im,XmptXt,ImminusXmptXt);
  for (i=0;i<m;i++)  {
    for (j=0;j<n;j++)  {
      ii = ((i+1+j*m) % n == 0) ?  ((int) (i+1+j*m)/n) : ((int) (i+1+j*m)/n) + 1 ;
      jj = i+1+j*m-(ii-1)*n;
      ii = ii-1;
      jj = jj-1;
      //printf("i=%d,j=%d,ii=%d,jj=%d\n",i,j,ii,jj);
      matrix_setpartofbigmatto0(1,m,ut);
      matrix_setpartofbigmatto0(1,n,vt);
      ut[0][ii] = 1.0;
      vt[0][jj] = 1.0;
      for (k=0;k<n;k++)  {
	InminusXtXmptjj[k][0] = InminusXtXmpt[k][jj];
	XmpXmptjj[k][0] = XmpXmpt[k][jj];
	Xmpii[k][0] = Xmp[k][ii];
      }
      for (l=0;l<m;l++)  {
	XmptXmpii[l][0] = XmptXmp[l][ii];
	ImminusXmptXtii[l][0] = ImminusXmptXt[l][ii];
	Xmptjj[l][0] = Xmpt[l][jj];
      }
      kronecker(1,m,m,m,ut,Im,kronutIm);
      kronecker(1,n,n,n,vt,In,kronvtIn);
      kronecker(1,m,n,n,ut,In,kronutIn);
      kronecker(m,m,1,n,Im,vt,kronImvt);
      matrix_mult(m,m*m,m*n,kronutIm,dXmptXmp,kronutImdXmptXmp);
      matrix_mult(n,n*n,m*n,kronvtIn,dXtXmpt,kronvtIndXtXmpt);
      matrix_mult(m,m*m,m*n,kronutIm,dXmptXt,kronutImdXmptXt);
      matrix_mult(n,n*n,m*n,kronvtIn,dXmpXmpt,kronvtIndXmpXmpt);
      matrix_mult(n,m*n,m*n,kronutIn,dXmp,kronutIndXmp);
      matrix_mult(m,m*n,m*n,kronImvt,dXmp,kronImvtdXmp);
      kronecker(n,1,m,m*n,InminusXtXmptjj,kronutImdXmptXmp,mat1);
      kronecker(n,n*m,m,1,kronvtIndXtXmpt,XmptXmpii,mat2);
      kronecker(n,1,m,m*n,XmpXmpt,kronutImdXmptXt,mat3);
      kronecker(n,n*m,m,1,kronvtIndXmpXmpt,ImminusXmptXtii,mat4);
      kronecker(n,m*n,m,1,kronutIndXmp,Xmptjj,mat5);
      kronecker(n,1,m,m*n,Xmpii,kronImvtdXmp,mat6);
      //printf("At element (%d,%d)\n",i,j);
      //bmp(m*n,m*n,mat1,"mat1");
      //bmp(n,n,InminusXtXmpt,"InminusXtXmpt");
      //bmp(1,m,ut,"ut");
      //bmp(m,m,Im,"Im");
      //bmp(m,m*m,kronutIm,"kronutIm");
      //bmp(m*m,m*n,kronXmptImKnmpluskronImXmpt,"kronXmptImKnmpluskronImXmpt");
      //bmp(m*n,m*n,dXmp,"dXmp");
      //bmp(m*m,m*n,dXmptXmp,"dXmptXmp");
      //bmp(n,1,InminusXtXmptjj,"InminusXtXmptjj");
      //bmp(m,m*n,kronutImdXmptXmp,"kronutImdXmptXmp");
      //bmp(m*n,m*n,mat2,"mat2");
      //bmp(m*n,m*n,mat3,"mat3");
      //bmp(m*n,m*n,mat4,"mat4");
      //bmp(m*n,m*n,mat5,"mat5");
      //bmp(n,m*n,kronutIn,"kronutIn");
      //bmp(n,m*n,kronutIndXmp,"kronutIndXmp");
      //bmp(m,1,Xmptjj,"Xmptjj");
      //bmp(m*n,m*n,mat6,"mat6");
      //bmp(n,1,Xmpii,"Xmpii");
      //bmp(m,m*n,kronImvtdXmp,"kronImvtdXmp");
      for (k=0;k<m*n;k++)  {
	for (l=0;l<m*n;l++)  { 
	  d2Xmp[i+j*m][k][l] = mat1[k][l]-mat2[k][l]-mat3[k][l]+mat4[k][l]-mat5[k][l]-mat6[k][l];
	  //printf("For entry %d, element (k=%d,l=%d) is %f\n",i+j*m,k,l,d2Xmp[i+j*m][k][l]);
	}
      }
    }
  }
  //print3darray(m*n,m*n,m*n,d2Xmp);
}

// ip calculates the inner produce of two vectors
// x,y a n length vector
// output, a scalar sum_{i=1}^n x[i]*y[i];

double ip(int n, double *x, double *y)
{  
  int i;
  double out;
  
  for(i=0,out=0.0;i<n;i++)  
    out += x[i]*y[i];
  return out;
}

// vec3darray takes a 3d array and unvecs it by row 
// m subjects each with i-th subject having a matrix n[i] by o[i]
// since the matrix sizes may be different n and o can be vectors of length m with the different dimensions
// if lengthn/lengtho are 1 then this indicates the dimension is the same for all m
// matrix_unvec3darray takes a 3d array and unnvecs it by row 
void matrix_vec3darray(int m, int *n, int *o, int lengthn, int lengtho, int maxn, int maxo, double X[m][maxn][maxo], double *Y)
{  
  int i,j,k,tmp;
  int nvec[m],ovec[m];
  for (i=0;i<m;i++)  {
    nvec[i]=n[i];
    ovec[i]=o[i];
  }
  
  //ivp(m,nvec,"nvec");
  //ivp(m,ovec,"ovec");
  
  // if n or o are scalars
  if (lengthn==1)  {
    for (i=0;i<m;i++)
      nvec[i] = n[0];
  }
  //ivp(m,ovec,"ovec");
  //printf("o[0]=%d,m=%d.length0=%d\n",o[0],m,lengtho);
  if (lengtho==1)  {
    for (i=0;i<m;i++)
      ovec[i] = o[0];
  }
  //return;
  //ivp(m,ovec,"ovec");
  //ivp(m,nvec,"nvec");
  //return;
  for (i=0;i<m;i++)  {
    if (i==0)  {
      tmp = 0;
    } else  {
      for (j=0,tmp=0;j<i;j++) 
	tmp += nvec[j]*ovec[j];
    }
    for (k=0;k<ovec[i];k++)  {
      for (j=0;j<nvec[i];j++)  {
	Y[tmp+(j+k*nvec[i])]=X[i][j][k];
	//printf("i=%d,j=%d,k=%d,tmp=%d,ovec[i]=%d,nvec[i]=%d,tmp+(j+k*o[i])=%d,Y[tmp+(j+k*nvec[i])]=%f\n",i,j,k,tmp,
	//ovec[i],nvec[i],tmp+(j+k*nvec[i]),Y[tmp+(j+k*nvec[i])]);
	
      }
    }
  }
}

void Xmphesswrapper(int *m, int *n, double *X, double *Y)
{  
  double X_m[NMAX][NMAX],Y_m[*m**n][*m**n][*m**n];
  int mn[1];
  mn[0]=*m**n;
  
  matrix_unvec(*m,*n,X,X_m);
  Xmphess(*m,*n,X_m,Y_m);
  //print3darray(mn[0],mn[0],mn[0],Y_m);
  matrix_vec3darray(*mn,mn,mn,1,1,*mn,*mn,Y_m,Y);
}

// matrix_unvec3darray takes a 3d array and unnvecs it by row 
void matrix_unvec3darray(int m, int *n, int *o, int lengthn, int lengtho, int maxn, int maxo, double *Y, double X[m][maxn][maxo])
{  
  int i,j,k,tmp;
  int nvec[m],ovec[m];
  for (i=0;i<m;i++)  {
    nvec[i]=n[i];
    ovec[i]=o[i];
  }
  
  //ivp(m,nvec,"nvec");
  //ivp(m,ovec,"ovec");
  
  // if n or o are scalars
  if (lengthn==1)  {
    for (i=0;i<m;i++)
      nvec[i] = n[0];
  }
  //ivp(m,ovec,"ovec");
  //printf("o[0]=%d,m=%d.length0=%d\n",o[0],m,lengtho);
  //return;
  if (lengtho==1)  {
    for (i=0;i<m;i++)
      ovec[i] = o[0];
  }
  //return;
  //ivp(m,ovec,"ovec");
  //ivp(m,nvec,"nvec");
  //return;
  for (i=0;i<m;i++)  {
    if (i==0)  {
      tmp = 0;
    } else  {
      for (j=0,tmp=0;j<i;j++) 
	tmp += nvec[j]*ovec[j];
    }
    //return;
    for (k=0;k<ovec[i];k++)  {
      for (j=0;j<nvec[i];j++)  {
	//printf("i=%d,j=%d,k=%d,tmp=%d,ovec[i]=%d,nvec[i]=%d,tmp+(j+k*o[i])=%d,Y[tmp+(j+k*nvec[i])]=%f\n",i,j,k,tmp,
	//       ovec[i],nvec[i],tmp+(j+k*nvec[i]),Y[tmp+(j+k*nvec[i])]);
	X[i][j][k]=Y[tmp+(j+k*nvec[i])];
      }
    }
  }
}

// ltmat takes a vector of length q(q+1)/2 stacks it by row in the lower triangular portion of the q by q lower triangular matrix M 
// m is a q(q+1)/2 by 1 length vector
void ltmat(int q, double *m, double out[NMAX][NMAX])
{  
  int i,j,count=0;
  for (j=0;j<q;j++)  {
    for (i=0;i<q;i++)  {
      if (i>=j)  {
	//printf("Before --> i=%d,j=%d,count=%d,m[count]=%f\n",i,j,count,m[count]);
	out[i][j] = m[count];
	count++;
	//printf("After --> i=%d,j=%d,count=%d,m[count]=%f\n",i,j,count,m[count]);
      } else {
	out[i][j] = 0.0;
      }
    }
  }
}

void dupmat( int q, double out[NMAX][NMAX])  
{
  int i,j,r,s;
  double tmp[q*(q+1)/2],matvec[q*q];
  
  matrix_setpartofbigmatto0(q*q,q*(q+1)/2,out);
  for (i=0;i<q;i++)  {
    for (j=0;j<i+1;j++)  {
      setvectorto0(q*(q+1)/2,tmp);
      tmp[j*q+i-(j+1)*j/2] = 1;
      setvectorto0(q*q,matvec);
      if (i!=j) { 
	matvec[i+j*q] = 1; 
	matvec[j+i*q] = 1; 
      } else {
	matvec[i+j*q] = 1; 
      }
      for (r=0;r<q*q;r++)  {
	for (s=0;s<q*(q+1)/2;s++)  
	  out[r][s] += matvec[r]*tmp[s];
      }
    }
  }
}

void dupltmat(int q, double out[NMAX][NMAX])  
{
  
  int i,j,k;
  dupmat(q,out);
  for (i=0;i<q;i++)  {
    for (j=0;j<q;j++)  {
      if (i<j)  {
	for (k=0;k<q*q+1/2;k++)
	  out[i+q*j][k] = 0;
      }
    }
  }
}

void dupmatwrapper(int *q, double *out)
{  
  double out_m[NMAX][NMAX];
  
  dupmat(*q,out_m);
  matrix_vec(*q**q,*q*(*q+1)/2,out_m,out);
}

void dupltmatwrapper(int *q, double *out)
{  
  double out_m[NMAX][NMAX];
  
  dupltmat(*q,out_m);
  matrix_vec(*q**q,*q*(*q+1)/2,out_m,out);
}
    
void outercrossprodltmatgrad(int q, double M[NMAX][NMAX], double out[NMAX][NMAX])  
{
  double Iq[NMAX][NMAX],MkronIq[NMAX][NMAX],IqkronM[NMAX][NMAX],Kqq[NMAX][NMAX],IqkronMKqq[NMAX][NMAX],Jq[NMAX][NMAX],MkronIqplusIqkronMKqq[NMAX][NMAX];
  
  identity(q,Iq);
  kronecker(q,q,q,q,M,Iq,MkronIq);
  kronecker(q,q,q,q,Iq,M,IqkronM); 
  commutmat(q,q,Kqq);
  matrix_mult(q*q,q*q,q*q,IqkronM,Kqq,IqkronMKqq);
  matrix_addscale(q*q,q*q,1.0,1.0,MkronIq,IqkronMKqq,MkronIqplusIqkronMKqq);
  dupltmat(q,Jq);
  matrix_mult(q*q,q*q,q*q,MkronIqplusIqkronMKqq,Jq,out);
}  

void outercrossprodltmathess(int q, double out[q*q][q*(q+1)/2][q*(q+1)/2])  
{
  int i,j,k,l;
  double Iq[NMAX][NMAX],Jq[NMAX][NMAX],Jqt[NMAX][NMAX],E[NMAX][NMAX],IqkronE[NMAX][NMAX],JqtIqkronE[NMAX][NMAX],JqtIqkronEJq[NMAX][NMAX];
  
  dupltmat(q,Jq);
  //bmp(q*q,q*(q+1)/2,Jq,"Jq");
  identity(q,Iq);
  matrix_transposetomat(q*q,q*(q+1)/2,Jq,Jqt);
  for (i=0;i<q;i++)  {
    for (j=0;j<q;j++)  {
      matrix_setpartofbigmatto0(q,q,E);
      E[i][j] += 1;
      E[j][i] += 1;
      kronecker(q,q,q,q,Iq,E,IqkronE);
      matrix_mult(q*(q+1)/2,q*q,q*q,Jqt,IqkronE,JqtIqkronE);
      matrix_mult(q*(q+1)/2,q*q,q*(q+1)/2,JqtIqkronE,Jq,JqtIqkronEJq);
      for (k=0;k<q*(q+1)/2;k++)  {
	for (l=0;l<q*(q+1)/2;l++)  {
	  out[i+q*j][k][l] = JqtIqkronEJq[k][l];
	}
      }
    }
  }
  //print3darray(q*q,q*(q+1)/2,q*(q+1)/2,out);
}

void outercrossprodltmatgradwrapper(int *q, double *M, double *out)
{  
  double M_m[NMAX][NMAX],out_m[NMAX][NMAX];
  
  matrix_unvec(*q,*q,M,M_m);
  outercrossprodltmatgrad(*q,M_m,out_m);
  matrix_vec(*q**q,*q*(*q+1)/2,out_m,out);
}

void outercrossprodltmathesswrapper(int *q, double *out)
{  
  
  int num[1];
  num[0] = *q*(*q+1)/2;
  double out_v[*q**q][*num][*num];
  outercrossprodltmathess(*q,out_v);
  //print3darray(*q**q,num[0],num[0],out_v);
  matrix_vec3darray(*q**q,num,num,1,1,num[0],num[0],out_v,out);
}

void ltvec (int q, double M[NMAX][NMAX], double m[q*(q+1)/2])  {
 
  int i,j,ind;
  ind = 0;
  for (j=0;j<q;j++)  {
    for (i=0;i<q;i++)  {
      if (i>=j)  {
	m[ind] = M[i][j];
	ind++;
      }
    }
  }
}
      

void opmpgradltmat (int q, double M[NMAX][NMAX], double out[NMAX][NMAX])
{

  double Mt[NMAX][NMAX],D[NMAX][NMAX],Dmpgrad[NMAX][NMAX],dMMt[NMAX][NMAX];

  matrix_transposetomat(q,q,M,Mt);
  matrix_crossprod(q,q,Mt,D);
  Xmpgrad(q,q,D,Dmpgrad);
  outercrossprodltmatgrad(q,M,dMMt);
  //bmp(q*q,q*q,Dmpgrad,"Dmpgrad");
  //bmp(q*q,q*(q+1)/2,dMMt,"dMMt");
  matrix_mult(q*q,q*q,q*(q+1)/2,Dmpgrad,dMMt,out);
}

void opmpgradltmatwrapper (int *q, double *M, double *out)
{  
  double M_m[NMAX][NMAX],out_m[NMAX][NMAX];
  
  matrix_unvec(*q,*q,M,M_m);
  opmpgradltmat(*q,M_m,out_m);
  matrix_vec(*q**q,*q*(*q+1)/2,out_m,out);
}


// quadprod evaluates the quadratic product of function
// m by m A, m by n X
// computes n by n t(X)%*%A%*%A
void quadprod (int m, int n, double X[NMAX][NMAX], double A[NMAX][NMAX], double out[NMAX][NMAX])
{

  double Xt[NMAX][NMAX],XtA[NMAX][NMAX];

  matrix_transposetomat(m,n,X,Xt);
  matrix_mult(n,m,m,Xt,A,XtA);
  matrix_mult(n,m,n,XtA,X,out);
}


void opmphessltmat (int q, double M[NMAX][NMAX], double out[q*q][q*(q+1)/2][q*(q+1)/2])
{

  double D[NMAX][NMAX],Dmpgrad[NMAX][NMAX],dMMt[NMAX][NMAX],d2MMt[q*q][q*(q+1)/2][q*(q+1)/2],tmp1[NMAX][NMAX];
  double Dmphess[q*q][q*q][q*q],tmp2[NMAX][NMAX],Mt[NMAX][NMAX],tmp;
  int i,j,k,l,r,s;



  outercrossprodltmatgrad(q,M,dMMt);
  outercrossprodltmathess(q,d2MMt);
  matrix_transposetomat(q,q,M,Mt);
  matrix_crossprod(q,q,Mt,D);
  //bmp(q,q,D,"D");
  Xmpgrad(q,q,D,Dmpgrad);
  Xmphess(q,q,D,Dmphess);
  //bmp(q*q,q*q,Dmpgrad,"Dmpgrad");
  //print3darray(q*q,q*q,q*q,Dmphess);
  for (i=0;i<q;i++)  {
    for (j=0;j<q;j++)  {
      matrix_setpartofbigmatto0(q*(q+1)/2,q*(q+1)/2,tmp1);
      for (k=0;k<q;k++)  {
	for (l=0;l<q;l++)  {
      	  for (r=0;r<q*(q+1)/2;r++)  {
	    for (s=0;s<q*(q+1)/2;s++)  
	      tmp1[r][s] += Dmpgrad[i+q*j][k+q*l]*d2MMt[k+q*l][r][s];
	  }
	}
      }
      //bmp(q*(q+1)/2,q*(q+1)/2,tmp1,"tmp1");
      //bmp(q*q,q*(q+1)/2,dMMt,"dMMt");
      for (r=0;r<q*(q+1)/2;r++)  {
	for (s=0;s<q*(q+1)/2;s++)  {
	  for (k=0,tmp=0;k<q*q;k++)  {
	    for (l=0;l<q*q;l++)  {
	      tmp += dMMt[k][r]*Dmphess[i+q*j][k][l]*dMMt[l][s];
	    }
	  }
	  tmp2[r][s] = tmp;
	}
      }
      //bmp(q*(q+1)/2,q*(q+1)/2,tmp2,"tmp2");
      for (r=0;r<q*(q+1)/2;r++)  {
	for (s=0;s<q*(q+1)/2;s++)  
	  out[i+q*j][r][s] = tmp1[r][s]+tmp2[r][s];
      }
    }
  }
}      
		    
		    
void opmphessltmatwrapper (int *q, double *M, double *out)
{  
  double M_m[NMAX][NMAX],out_a[*q**q][*q*(*q+1)/2][*q*(*q+1)/2];
  int qq[1];
  qq[0]=*q*(*q+1)/2;
  
  matrix_unvec(*q,*q,M,M_m);
  opmphessltmat(*q,M_m,out_a);
  matrix_vec3darray(*q**q,qq,qq,1,1,*qq,*qq,out_a,out);
}
      
      
				
				
      



