


library(MASS)
library(nlme)
library(lmm)
##library(lme4)
##library(lmm,lib="/netscr/rigarcia/lmeq/current")
##library(numDeriv,lib="/netscr/rigarcia/lmeq/current")
##dyn.load("pmeqpl.dll")
##to compile R CMD SHLIB lmeqpl.c
dyn.load("lmeqpl.so")
source("lmeqpl.r")

## fits models for simulations see lmeqfitcluster.r

lmeqfit <- function(data,input,control,modelnames)  {
  y <- data$y
  X <- data$X
  Z <- data$Z
  n <- data$n
  ni <- data$ni
  p <- data$p
  q <- data$q
  noscale <- rep(1,p+q*(q+1)/2)
  nsim <- input$nsim
  penalize <- input$penalize
  out <- list()
  for ( model in modelnames )  {
    penalty.fun <- input[[model]][["penalty.fun"]]
    penalty.start <- input[[model]][["penalty.start"]]
    out.mle <- lmeq.plm(y,X,Z,n,ni,p,q,0,0,penalty.fun,noscale,penalize,control)$estimate
    scale <- groupdistance(out.mle$theta,c(1:p,groupltmatbyrow(q)+p),p+q)
    scale1 <- sqrt(scale$lengrp)
    scale2 <- sqrt(scale$lengrp)/scale$distgrp
    scale3 <- 1/scale$distgrp
    if (model=="lasso") scaletheta <- scale1
    if (model=="alasso1") scaletheta <- scale2
    if (model=="alasso2") scaletheta <- scale3
    if (model=="scad") scaletheta <- scale1
    outy<- lmeq.plm(y,X,Z,n,ni,p,q,penalty.start[1],penalty.start[2],penalty.fun,scaletheta,penalize,control)$estimate
    cat(paste("Started computing RE penalty estimation for model",model,"at",date()),"\n")
    ##penalty.re <- lmeq.plm.re(y,X,Z,n,ni,p,q,penalty.start[1],penalty.start[2],penalty.fun,scaletheta,penalize,nsim,control)$estimate$penalty
    penalty.re <- c(penalty.start)	
    out.re <- lmeq.plm(y,X,Z,n,ni,p,q,penalty.re[1],penalty.re[2],penalty.fun,scaletheta,penalize,control)$estimate
    cat(paste("Started computing ICQ penalty estimation for model",model,"at",date()),"\n")
    out.val.re.icq <- lmeq.icq(y,X,Z,n,ni,p,q,penalty.re[1],penalty.re[2],penalty.fun,scaletheta,penalize,lmeq.bicpenalty,control)
    penalty.icq <- lmeq.plm.icq(y,X,Z,n,ni,p,q,penalty.re[1],penalty.re[2],penalty.fun,scaletheta,penalize,lmeq.bicpenalty,control)$estimate$penalty
    out.icq <- lmeq.plm(y,X,Z,n,ni,p,q,penalty.icq[1],penalty.icq[2],penalty.fun,scaletheta,penalize,control)$estimate
    out.val.icq.icq <- lmeq.icq(y,X,Z,n,ni,p,q,penalty.icq[1],penalty.icq[2],penalty.fun,scaletheta,penalize,lmeq.bicpenalty,control)
    out.model <- list(re=list(theta=out.re$theta,penalty=penalty.re,sigma2=out.re$sigma2,icq=out.val.re.icq),
                      icq=list(theta=out.icq$theta,penalty=penalty.icq,sigma2=out.icq$sigma2,icq=out.val.icq.icq))
    out[[model]] <- out.model
  }
  out.mle <- lmeq.plm(y,X,Z,n,ni,p,q,0,0,alasso,noscale,penalize,control)$estimate
  out.val.mle.icq <- lmeq.icq(y,X,Z,n,ni,p,q,0,0,alasso,noscale,penalize,lmeq.bicpenalty,control)	
  out <- c(list(mle=list(theta=out.mle$theta,sigma2=out.mle$sigma2,icq=out.val.mle.icq)),out)
}

lmeqoverallfit <- function(datasets,input,control)  {
  n <- length(datasets)
  fun <- function(i)  {
    cat(paste(paste("dataset",i),date(),"\n"))
    ##debug(lmeqfit)
    ##lmeqfit(datasets[[i]],input,control,c("lasso","alasso1","alasso2","scad"))
    lmeqfit(datasets[[i]],input,control,c("lasso","alasso1","scad"))
  }
  out <- lapply(1:n,fun)
  out 
}


## control <- list(relativeerror=.05,maxiter=33)
## datafilenames <- c("datan50sigma21.rdata","datan50sigma23.rdata")

## ## following penalty parameters values estimated from initial simulations
## penalty.start.scad <- list(c(0,.1),c(0,.05))
## penalty.start.alasso <- list(c(0,1),c(0,.1))
## names(penalty.start.scad) <- names(penalty.start.alasso) <- datafilenames
## p <- q <- 8
## penalize <- c(rep(FALSE,p),rep(TRUE,q*(q+1)/2))
## for (model in datafilenames)  {
##   cat("Estimating model",model,"at",date(),"\n")
##   input.alasso <- list(penalty.fun=alasso,scalemle=TRUE,penalty.start=penalty.start.alasso[[model]])
##   input.scad <- list(penalty.fun=scad,scalemle=FALSE,penalty.start=penalty.start.scad[[model]])
##   input <- list(alasso=input.alasso,scad=input.scad,nsim=500,penalize=penalize)
##   load(model)
##   out <- lmeqoverallfit(datasets[1],input,control)
##   save(out,file=paste("out",Sys.Date(),model,sep=""))
## }
