library(lmm)
source("penaltyfunctions.r")
source("pl.r")
source("lmeq.r")

## lmeq1profgrouppenloglikest computes maximzation of penalized log likelihood function of LMM
lmeq1profgrouppenloglikest <- function(theta=NULL,penalty,penalty.fun,groupindex,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  p <- nrow(theta)
  ll <- loglik <- lmeq1
  ll.grad <- attr(loglik,"gradient")
  ll.hess <- attr(loglik,"hessian")
  pf.fun <- penalty.fun
  pf.grad <- attr(penalty.fun,"gradient")
  nexttheta.mle <- nexttheta.mle.lmeq1
  nexttheta.lqa <- attr(penalty.fun,"nexttheta.grouplqa.lmeq1")
  nexttheta.lla <- attr(penalty.fun,"nexttheta.grouplla.lmeq1")
  ##lets extract n sample size and which coefficients we are penalizing
  numgroup <- length(unique(groupindex))
  n <- ldots[["n"]]
  ni <- ldots[["ni"]]
  ##if penalty paramter is zero  
  if (is.null(ldots[["penalize"]])) penalize <- rep(TRUE,p) else penalize <- ldots[["penalize"]]
  ldots <- c(ldots,list(groupindex=groupindex,numgroup=numgroup,penalize=penalize,penalty=penalty))
  formals(ll)[-1] <- ldots[names(formals(ll))[-1]]
  formals(ll.grad)[-1] <- ldots[names(formals(ll.grad))[-1]]
  formals(ll.hess)[-1] <- ldots[names(formals(ll.hess))[-1]]
  formals(pf.fun)[-1] <- ldots[names(formals(pf.fun))[-1]]
  formals(pf.grad)[-1] <- ldots[names(formals(pf.grad))[-1]]
  formals(nexttheta.mle)[-1] <- ldots[names(formals(nexttheta.mle))[-1]]
  formals(nexttheta.lqa)[-1] <- ldots[names(formals(nexttheta.lqa))[-1]]
  formals(nexttheta.lla)[-1] <- ldots[names(formals(nexttheta.lla))[-1]]
  ##lets create penalized loglikelihood function
  ###  mlefun computes unpenalized mle 
  ##nexttheta.lla computes local linear approximation, Zou and Li (2007) Annals
  #control$mle <- control
  #control$maxiter <- 5
  ls.est <- theta
  if (!is.null(control$start)) ls.est <- control$start$theta 
  mlefun <- nexttheta.mle
  mle.est <- iterate(ls.est,mlefun,control)$estimate
  ##if penalty == 0 return unpenalized
  if (all(penalty[penalize]==0)) {
    maxima <- cbind(ls.est,mle.est)
    funval <- sapply(1:ncol(maxima),function(i) ll(maxima[,i,drop=FALSE]))
    out <- maxima[,which.max(funval),drop=FALSE]
    return(out)
  }
  lqa.est <- iterate(mle.est,nexttheta.lqa,control)$estimate
  startval <- cbind(ls.est,mle.est,lqa.est)
  ##lets creat different starting values off of lqa/starting value
  ##for (i in 1:nrow(mle.est)) {
  ##  tmp <- mle.est
  ##  tmp[i,] <- 0
  ##  startval <- cbind(startval,tmp)
  ##}
  ##lets employ lla at these different starting values
  ##debug(nexttheta.grouplla)
  maxima <- sapply(1:ncol(startval),function(i) iterate(startval[,i,drop=FALSE],nexttheta.lla,control)$estimate)
  ##lets also include ls, mle, and lqa estimates as possible maxima
  maxima <- cbind(maxima,startval)
  ##note I always use the original function not approximations to determine which is maximum value
  pl <- function(theta) {
    gd <- groupdistance(theta,ldots$groupindex,ldots$numgroup)
    ##fun <- function(i) pf.fun(sqrt(gd$lengrp[i])*gd$distgrp[i],penalty=penalty[i])/gd$lengrp[i]
    fun <- function(i) pf.fun(sqrt(1)*gd$distgrp[i],penalty=penalty[i])/gd$lengrp[i]
    out <- ll(theta) - sum(n)*sum(sapply(1:p,fun)[penalize])
    out
  }
  funval <- sapply(1:ncol(maxima),function(i) pl(maxima[,i,drop=FALSE]))
  out <- maxima[,which.max(funval),drop=FALSE]
  ##cat("At max\n")
  ##browser()
  out
}

## computes E(b|y) of LMM
lmeq.plm.b <- function(b,theta,sigma2,y,X,Z,n,p,q) {
  out <- red.r(theta,sigma2,y,X,Z,n,p,q)
  out
}

## computes conditional estimate of variance of LMM
lmeq.plm.sigma2 <- function(sigma2,theta,y,X,Z,b,n,p,q,ni)  {
  beta <- theta[1:p,1,drop=FALSE]
  m <- theta[(p+1):(p+q*(q+1)/2),1,drop=FALSE]
  M <- ltmat(m)
  b.mean <- b$mean
  b.var <- b$var
  fun <- function(i)  {
    out <- crossprod(y[[i]]-X[[i]]%*%beta-Z[[i]]%*%M%*%b.mean[[i]])+sum(diag(Z[[i]]%*%M%*%b.var[[i]]%*%t(Z[[i]]%*%M)))
    out <- out
    out
  }
  ni <- sapply(y,nrow)
  out <- sapply(1:n,fun)
  out <- sum(out)/(sum(ni))
  ##out <- sum(out)/(sum(ni)-sum(theta!=0))
  out
}

lmeq.plm.theta <- function(theta,penalty,penalty.fun,groupindex,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  ##debug(lmeq1profgrouppenloglikest)
  theta <- lmeq1profgrouppenloglikest(theta,penalty,penalty.fun,groupindex,control,ldots)
  theta
}

## lmeq.plm computes MPL estimate of LMM
## y,X,Z,n,ni,p,q as in lmeq.r
## penalty.par.beta is penalty parameter of betas
## penalty.par.m is penalty parameter of random effects
## penalty.fun is type of penalty function
## penalize whether parameter is penalized or not
## control is as in simultcondest in pl.r
## ... additional arguments
## output as in simultcondest in pl.r
lmeq.plm <- function(y,X,Z,n,ni,p,q,penalty.par.beta,penalty.par.m,penalty.fun,scale,penalize,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  data <- do.call("rbind",lapply(1:n,function(i) data.frame(id=i,y=y[[i]],X=X[[i]],Z=Z[[i]])))
  colnames(data) <- c("id","y",paste("X",1:(p+q),sep=""))
  if (is.null(control$start)) {
    out.mle <- ecmeml.lmm(as.numeric(data$y),as.numeric(data$id),as.matrix(data[,paste("X",1:(p+q),sep="")]),1:p,(p+1):(p+q))
    beta <- matrix(out.mle$beta,,1)
    sigma2 <- out.mle$sigma2
    m <- matrix(ltvec(t(chol(out.mle$psi))),,1)
    theta <- matrix(rbind(beta,m),,1)	
  }  else {
    theta <- control$start$theta
    sigma2 <- control$start$sigma2
  }
  # lets dump arguments ant get them ready for startlist
  penalty.par <- c(penalty.par.beta*scale[1:p],penalty.par.m*scale[(p+1):(p+q*(q+1)/2)])
  groupindex <- c(1:p,groupltmatbyrow(q)+p)
  numgroup <- length(unique(groupindex))
  args <- list(y=y,X=X,Z=Z,penalty=penalty.par,
               penalty.fun=penalty.fun,p=p,q=q,n=n,ni=ni,groupindex=groupindex,numgroup=numgroup,penalize=penalize,control=control)
  args <- c(args,ldots)
  ##debug(lmeq.plm.beta)
  # lets dump arguments ant get them ready for startlist
  startlist <- list(b=NULL,theta=theta,sigma2=sigma2)
  funlist <- list(lmeq.plm.b,lmeq.plm.theta,lmeq.plm.sigma2)
  control$which.convergence <- c(FALSE,TRUE,FALSE) # determines which parameters want to be small
  out <- simultcondest(startlist,funlist,args,control)
  out
}

lmeq.bicpenalty <- function(theta,ni)  {
  n <- sum(ni)
  df <- sum(unlist(theta)!=0) 
  out <- list(aic=2*df,bic=df*log(n))
  out <- out$bic
  out
}

## lmeq.plm.icq computes ICQ penalty estimate of LMM
## arguments as in lmeq.plm
## modelpenalty where ICQ = -2*Qfun(theta|theta(s)) + modelpenalty(theta) 
## output as in simultcondest in pl.r
lmeq.plm.icq <- function(y,X,Z,n,ni,p,q,penalty.par.beta,penalty.par.m,penalty.fun,scale,penalize,modelpenalty,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  out.mle <- lmeq.plm(y,X,Z,n,ni,p,q,0,0,penalty.fun,scale,penalize,control,ldots)$estimate
  # lets dump arguments and get them ready for startlist
  fun <- function(penalty)  {
    out <- lmeq.plm(y,X,Z,n,ni,p,q,penalty[1],penalty[2],penalty.fun,scale,penalize,control,ldots)$estimate
    tmpargs <- list(theta=out$theta,sigma2=out$sigma2,y=y,X=X,Z=Z,b=out.mle$b,n=n,p=p,q=q,ni=ni)
    qfun <- lmeq1.function
    formals(modelpenalty) <- tmpargs[names(formals(modelpenalty))]
    formals(qfun) <- tmpargs[names(formals(qfun))]
    out <- -2*qfun()+modelpenalty()
    ##cat("qfun=",qfun(),"modelpenalty=",modelpenalty(),"out=",out,"\n")
    out
  }
  control$plot <- FALSE
  out <- icq2penaltyest(fun,c(penalty.par.beta,penalty.par.m),5,1.5,3,control)
  out
}

lmeq.icq <- function(y,X,Z,n,ni,p,q,penalty.par.beta,penalty.par.m,penalty.fun,scale,penalize,modelpenalty,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  out.mle <- lmeq.plm(y,X,Z,n,ni,p,q,0,0,penalty.fun,scale,penalize,control,ldots)$estimate
  # lets dump arguments and get them ready for startlist
  fun <- function(penalty)  {
    out <- lmeq.plm(y,X,Z,n,ni,p,q,penalty[1],penalty[2],penalty.fun,scale,penalize,control,ldots)$estimate
    tmpargs <- list(theta=out$theta,sigma2=out$sigma2,y=y,X=X,Z=Z,b=out.mle$b,n=n,p=p,q=q,ni=ni)
    qfun <- lmeq1.function
    formals(modelpenalty) <- tmpargs[names(formals(modelpenalty))]
    formals(qfun) <- tmpargs[names(formals(qfun))]
##    cat("qfun(theta)=",qfun(theta=out$theta,sigma2=out.mle$sigma2),"modelpenalty(theta)=",modelpenalty(theta=out$theta),
##        "ICQ(theta)=",-2*qfun(theta=out$theta,sigma2=out.mle$sigma2)+modelpenalty(theta=out$theta),"\n")
##    cat("qfun(mle)=",qfun(theta=out.mle$theta,sigma2=out.mle$sigma2),"modelpenalty(mle)=",modelpenalty(theta=out.mle$theta),
##        "ICQ(mle)=",-2*qfun(theta=out.mle$theta,sigma2=out.mle$sigma2)+modelpenalty(theta=out.mle$theta),"\n")
    out <- -2*qfun()+modelpenalty()
    ##xcat("qfun=",qfun(),"modelpenalty=",modelpenalty(),"out=",out,"\n")
    out
  }
  out <- fun(c(penalty.par.beta,penalty.par.m))
  out
}

penaltyreest1 <- function(penalty,penalty.fun,betas,n,p,nsim,penalize,scalebeta,groupindex,numgroup)  {
  betas <- do.call("rbind",lapply(1:nsim,function(i) groupdistance(matrix(unlist(betas[i,]),,1),groupindex,numgroup)$distgrp))
  lengrp <- groupdistance(matrix(unlist(betas[1,]),,1),groupindex,numgroup)$lengrp
  if (attr(penalty.fun,"name")=="alasso")  {
    betas <- sweep(betas,2,FUN="*",scalebeta/lengrp)
    out <- apply(betas,2,mean)
    out <- sum(out)
    out <- 1/out/n
    return(out)
  }
  fun <- function(penalty)  {
    fun1 <- function(beta)  {
      fun2 <- function(i) attr(penalty.fun,"density")(beta[i],n=n,penalty=penalty*scalebeta[i],keepnc=TRUE)/lengrp[i]
      out <- sum(sapply(1:p,fun2)[penalize])
      out
    }
    out <- mean(apply(betas,1,fun1))
    out
  }
  mult <- 10
  lam <- exp(seq(log(penalty/mult),log(penalty*mult),length=5))
  if (FALSE)  {
    lam2 <- exp(seq(log(penalty/mult),log(penalty*mult),length=200))
    plot(lam2,sapply(lam2,fun))
    abline(v=min(abs(betas)))
  }
  lamlb <- min(abs(betas))
  fn <- function(start) optim(start,fun,method="L-BFGS-B",lower=lamlb,upper=max(lam),
                              control=list(fnscale=-1))$par
  maxima <- try(sapply(lam,fn))
  if (inherits(maxima, "try-error")) {
    fname <- paste("refailure",sample(1:100,1),".rdata",sep="")
    save(betas,fun,penalty,penalty.fun,nsim,penalize,scalebeta,fn,lam,penalty,file=fname)
    stop(paste("RE est failure see", fname))
  }
  out <- maxima[which.max(sapply(maxima,fun))]
  out
}

lmeq.plm.re.penalty <- function(penalty,scale,file.theta,penalty.fun,n,p,q,ni,nsim,penalize,groupindex)  {
  thetas <- read.table(file=file.theta)
  betas <- thetas[,1:p,drop=FALSE]
  ms <- thetas[,(p+1):(p+q*(q+1)/2),drop=FALSE]
  ind <- 1:p
  pen <- penalize[ind]
  scl <- scale[ind]
  numgroup.new <- length(unique(groupindex[ind]))
  groupindex.new <- recode(groupindex[ind],unique(groupindex[ind]),1:numgroup.new)
  if (any(pen))  {
    penalty.beta <- penaltyreest1(penalty[1],penalty.fun,betas,sum(n),p,nsim,pen,scl,groupindex.new,numgroup.new)
  } else penalty.beta <- 0
  ind <- (p+1):(p+q*(q+1)/2)
  pen <- penalize[ind]
  scl <- scale[ind]
  numgroup.new <- length(unique(groupindex[ind]))
  groupindex.new <- recode(groupindex[ind],unique(groupindex[ind]),1:numgroup.new)
  if (any(pen))  {
    penalty.m <- penaltyreest1(penalty[2],penalty.fun,ms,sum(n),q*(q+1)/2,nsim,pen,scl,groupindex.new,numgroup.new)
  } else penalty.m <- 0
  out <- c(penalty.beta,penalty.m)
  out
}

lmeq.plm.re.sim <- function(file.theta,file.b,sigma2,y,X,Z,n,p,q,ni,nsim,penalty,scale,groupindex,numgroup,penalize,penalty.fun)  {
  ##if (file.exists(file.theta)) file.remove(file.theta)
  ni <- sapply(y,nrow)
  k <- lapply(ni,function(x) rep(1,x))
  ##file.theta <- tempfile("filetheta")
  if (attr(penalty.fun,"name")=="scad") {
    dllname <- "lmme_sim_scad"
  } else dllname <- "lmme_sim_lasso"
  simtheta <- 1
  theta <- matrix(0,p+q*(q+1)/2)
  tmp <- c(penalty[1]*scale[1:p],penalty[2]*scale[(p+1):(p+q*(q+1)/2)])
  .C(dllname,as.double(theta),as.double(sigma2),as.double(unlist(y)),as.double(unlist(X)),as.double(unlist(Z)),
     as.double(unlist(k)),as.integer(n),as.integer(p),as.integer(q),as.integer(unlist(ni)),
     as.integer(nsim),as.character(file.b),as.character(file.theta),as.double(tmp),as.integer(groupindex-1),as.integer(numgroup),
     as.integer(penalize),as.integer(simtheta))
  file.theta
}

lmeq.plm.re.sigma2 <- function(sigma2,file.theta,file.b,y,X,Z,n,p,q,ni,nsim)  {
  thetas <- read.table(file.theta)
  bs <- read.table(file.b)
  bs <- lapply(1:nsim,function(i) matrix(bs[i,],n,q,byrow=TRUE))
  thetas <- lapply(1:nsim,function(i) matrix(unlist(thetas[i,]),,1))
  bs <- lapply(1:nsim,function(l) list(mean=lapply(1:n,function(j) matrix(unlist(bs[[l]][j,]),,1)),var=lapply(1:n,function(i) matrix(0,q,q))))
  fun <- function(i) {
    out <- lmeq.plm.sigma2(sigma2,thetas[[i]],y,X,Z,bs[[i]],n,p,q,ni)
    out
  }
  out <- sapply(1:nsim,fun)
  out <- mean(out)
  out
}

## lmeq.plm.recomputes random effects penalty estimate of LMM
## arguments as in lmeq.plm
## modelpenalty where ICQ = -2*Qfun(theta|theta(s)) + modelpenalty(theta) 
## output as in simultcondest in pl.r
lmeq.plm.re <- function(y,X,Z,n,ni,p,q,penalty.par.beta,penalty.par.m,penalty.fun,scale,penalize,nsim,control,...)  {
  if (is.list(c(...))) ldots <- c(...) else ldots <- list(...)
  groupindex <- c(1:p,groupltmatbyrow(q)+p)
  numgroup <- length(unique(groupindex))
  file.theta <- tempfile("filetheta")
  file.b <- tempfile("fileb")
  penalty <- c(penalty.par.beta,penalty.par.m)
  args <- c(list(y=y,X=X,Z=Z,penalty=penalty,scale=scale,penalty.fun=penalty.fun,p=p,q=q,n=n,ni=ni,groupindex=groupindex,
                 numgroup=numgroup,penalize=penalize,nsim=nsim,control=control,file.b=file.b),ldots)
  ##debug(lmeq.plm.beta)
  # lets dump arguments ant get them ready for startlist
  startlist <- list(file.theta=file.theta,sigma2=1,penalty=penalty)
  funlist <- list(lmeq.plm.re.sim,lmeq.plm.re.sigma2,lmeq.plm.re.penalty)
  control$which.convergence <- c(FALSE,FALSE,TRUE) # determines which parameters want to be small
  ##control$debug <- TRUE; ##control$whichdebug <- 1
  out <- simultcondest(startlist,funlist,args,control)
  out
}


