#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "lmeq.c"

// assumes model set up of all previous included c code  

// glmme_lpd_b_q1 lg density of kernel pertaining to random effect of Q1 function penalized Q function of GLMM
// to be used/passed within ARMS  

void glmme_lpd_b_q1(double *b, double *theta, double *phi, double *y, double *X, double *Z, double *k, int *n, int *p, int *q, int *ni, 
		  double (*conanfun)(double x), double (*bfun)(double x), double (*cfun)(double x, double *phi), double *out) 
{
  int i,j,subin;
  double eta,conan;

  double beta[*p],m[*q*(*q+1)/2],M[NMAX][NMAX],tmp;
  for (i=0;i<*p;i++)  
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)  
    m[i-*p] = theta[i];
  ltmat(*q,m,M);

  for (subin=0,out[0]=0.0;subin<*ni;subin++)  {
    for (j=0,eta=0.0;j<*p;j++)  
      eta += X[subin+*ni*j]*beta[j];
    for (j=0;j<*q;j++)  {
      for (i=0,tmp=0.0;i<*q;i++)
	tmp += M[j][i]*b[i];
      eta += Z[subin+*ni*j]*tmp;
    }
    conan = conanfun(eta);
    out[0] += k[subin]*(y[subin]*conan-bfun(conan))/(*phi)+cfun(y[subin],phi);
  }
}

// glmme_lpd_b_q2 lg density of kernel pertaining to random effect of Q2 function penalized Q function of GLMM
// to be used/passed within ARMS  

void glmme_lpd_b_q2(double *b, int *q, double *out) 
{
  int j;
  for (j=0,out[0]=0.0;j<*q;j++)  {
      out[0] += b[j]*b[j];
  }
  out[0] = -out[0]/2.0;
}

// glmme_lpd_b  log density of kernel pertaining to random effect of penalized Q function of GLMM
// to be passed to ARMS function

double glmme_lpd_b(double bj, void *fundata) {
  struct data_glmme *rod;
  rod = fundata;
  double out1[1],out2[1],out,*b;

  b = rod->b;
  b[*(rod->j)] = bj;
  rod->b = b;
  glmme_lpd_b_q1(rod->b,rod->theta,rod->phi,rod->y,rod->X,rod->Z,rod->k,rod->n,rod->p,rod->q,rod->ni,rod->conanfun,rod->bfun,rod->cfun,out1);
  glmme_lpd_b_q2(rod->b,rod->q,out2);
  out = out1[0]+out2[0];
  return(out);
}

// glmme_sim_b simulates one sample of density of random effect of penalized Q function of GLMM using ARMS

void glmme_sim_b(double *theta, double *b, double *phi, double *y, double *X, double *Z, double *k, 
		 int *n, int *p, int *q, int *ni, double (*conanfun)(double x), 
		 double (*bfun)(double x), double (*cfun)(double x, double *phi))
{

  struct data_glmme rod; 
  int nimax,sub;
  nimax=max_int(*n,ni);
  
  rod.theta = theta;
  rod.phi = phi;
  rod.n = n;
  rod.p = p;
  rod.q = q;
  rod.ni = ni;
  rod.conanfun = conanfun;
  rod.bfun = bfun;
  rod.cfun = cfun;

  int j,ninit=7,dometrop=1,subin,err,nii[1];
  double leftlimit=-5,rightlimit=5,bjprev[1],bjsamp[1],yi[nimax],Xi[nimax**p],Zi[nimax**q],ki[nimax],bi[*q];


  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*p;j++)  {
      for (subin=0;subin<ni[sub];subin++)  
	Xi[subin+ni[sub]*j] = X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)];
    }
    for (j=0;j<*q;j++)  {
      bi[j] = b[j+sub**q];
      for (subin=0;subin<ni[sub];subin++)  
	Zi[subin+ni[sub]*j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
    }
    for (subin=0;subin<ni[sub];subin++)  {  
      yi[subin] = y[(sub==0) ? subin : sum_int(sub,ni) + subin];
      ki[subin] = k[(sub==0) ? subin : sum_int(sub,ni) + subin];
    }
    nii[0] = ni[sub];
    rod.X = Xi;
    rod.Z = Zi;
    rod.y = yi;
    rod.k = ki;
    rod.ni = nii;
    rod.b = bi;
    //vp(*q,bsim[i],"bsim[i]");
    //vp(*q,rod.b,"rod.b");
    for (j=0;j<*q;j++)  {
      rod.j = &j;
      //printf("rod.j=%d,j=%d\n",(rod.j)[0],j);
      bjprev[0] = b[j+sub**q];
      //vp(*q,bsim[i],"bsim[i]");
	//vp(*q,rod.b,"rod.b");
	//printf("bjprev=%f,glmme_lpd_re=%f\n",bjprev[0],glmme_lpd_re(bjprev[0],&rod));
      err = arms_simple(ninit,&leftlimit,&rightlimit,glmme_lpd_b,&rod,dometrop,bjprev,bjsamp);
      //printf("bjsamp=%f,glmme_lpd_re=%f\n",bjsamp[0],glmme_lpd_re(bjsamp[0],&rod));
      b[j+sub**q] = bjsamp[0];
      rod.b[j] = bjsamp[0];
      //vp(*q,rod.b,"rod.b");
      //vp(*q,b,"b");
      if (err>0) {
	printf("error code = %d\n",err);
	exit(1);
      }
    }
  }
}

// glmme_lpd_theta_q1  log density of kernel pertaining to parameter (theta) of Q1 portion of penalized Q function of GLMM
// to be passed to ARMS function
// used when computing RE penalty estimate

void glmme_lpd_theta_q1(double *theta, double *phi, double *y, double *X, double *kronbitZiJqarray, double *k, int *n, int *p, int *q, int *ni,
			double (*conanfun)(double x), double (*bfun)(double x), double (*cfun)(double x, double *phi),
			int *whichj, double *etamat, double *out)
{
  // some preliminaries
  //printf("phi=%f\n",*phi);

  int i,j,sub,subin;

  double beta[*p],m[*q*(*q+1)/2];
  for (i=0;i<*p;i++)
    beta[i] = theta[i];
  for (i=*p;i<*p+*q*(*q+1)/2;i++)
    m[i-*p] = theta[i];

  double logliki[*n],eta,tmp,conan;
  
  j = whichj[0];

  setvectorto0(*n,logliki);
  for (sub=0;sub<*n;sub++)  {
    for (subin=0;subin<ni[sub];subin++)  {
      if (j>=*p)  {
	eta = kronbitZiJqarray[(sub==0) ? subin+ni[sub]*(j-*p) : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*(j-*p))]*theta[j];
	//printf("num1=%f,theta=%f,eta=%f\n",
	//       kronbitZiJqarray[(sub==0) ? subin+ni[sub]*(j-*p) : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*(j-*p))],theta[j],eta);
      }  else  {
	eta = X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)]*theta[j];
	//printf("num2=%f,theta=%f,eta%f\n",X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)],theta[j],eta);
      }
      eta += etamat[(sub==0) ? subin : sum_int(sub,ni) + subin];
      //printf("pphi=%f\n",*phi);
      //printf("eta=%f\n",eta);
      conan = conanfun(eta);
      //conan=eta;
      //printf("conan=%f\n",conan);
      //tmp = -(y[1]-eta)*(y[1]-eta)/2.0/phi[0];
      //out[0]=1;
      //continue;
      tmp = ((double) k[(sub==0) ? subin : sum_int(sub,ni) + subin])*(y[(sub==0) ? subin : sum_int(sub,ni) + subin]*conan-bfun(conan))/(*phi);
      //tmp += cfun(y[(sub==0) ? subin : sum_int(sub,ni) + subin],phi);
      //printf("A=%f,B=%f,C=%f\n",(double) karray[i][subin][0],(yarray[i][subin][0]*conan-bfun(conan))/(*phi),cfun(yarray[i][subin][0],phi));
      //printf("AA=%f,BB=%f\n",yarray[i][subin][0],*phi);
      logliki[sub] += tmp;
      //printf("logliki=%f\n",logliki[i]);
    }
  }
  out[0] = sum(*n,logliki);
}

// glmme_lpd_theta_lasso computes log density of kernel pertaining to parameter (theta) of penality portion of penalized Q function of GLMM
// to be passed to ARMS function

double glmme_lpd_theta_lasso(double thetaj, void *fundata) {
  struct data_glmme *rod;
  rod = fundata;
  double out1[1],out2[1],out,*theta;
  
  theta = rod->theta;
  theta[*(rod->j)] = thetaj;

  int inttmp[1];
  inttmp[0] = *(rod->p)+*(rod->q)*(*(rod->q)+1)/2.0;
  int sni[1];
  //  sni[0]=sum_int(*(rod->n),rod->ni);
  sni[0]=*(rod->n);
  glmme_lpd_theta_q1(rod->theta,rod->phi,rod->y,rod->X,rod->kronbitZiJqarray,rod->k,rod->n,rod->p,rod->q,rod->ni,rod->conanfun,
		     rod->bfun,rod->cfun,rod->j,rod->etamat,out1);
  lpd_lasso(theta,&(inttmp[0]),rod->penalty,sni,rod->groupindex,rod->numgroup,rod->penalize,out2);
  //printf("out1=%f,out2=%f\n",out1[0],out2[0]);
  out = out1[0]+out2[0];
  return(out);
}

// glmme_lpd_theta_scad computes log density of kernel pertaining to parameter (theta) of penality portion of penalized Q function of GLMM
// to be passed to ARMS function

double glmme_lpd_theta_scad(double thetaj, void *fundata) {
  struct data_glmme *rod;
  rod = fundata;
  double out1[1],out2[1],out,*theta;
  
  theta = rod->theta;
  theta[*(rod->j)] = thetaj;

  int inttmp[1];
  inttmp[0] = *(rod->p)+*(rod->q)*(*(rod->q)+1)/2.0;
  int sni[1];
  //sni[0]=sum_int(*(rod->n),rod->ni);
  sni[0]=*(rod->n);
  glmme_lpd_theta_q1(rod->theta,rod->phi,rod->y,rod->X,rod->kronbitZiJqarray,rod->k,rod->n,rod->p,rod->q,rod->ni,rod->conanfun,
		     rod->bfun,rod->cfun,rod->j,rod->etamat,out1);
  lpd_scad(theta,&(inttmp[0]),rod->penalty,sni,rod->groupindex,rod->numgroup,rod->penalize,out2);
  //printf("out1=%f,out2=%f\n",out1[0],out2[0]);
  out = out1[0]+out2[0];
  return(out);
}


// glmme_sim_theta_lasso simulates one sample of theta of density of penalized Q function 
// of random effect theta under the lasso penalty function
// To used when computing RE penalty estimate

void glmme_sim_theta_lasso(double *theta, double *b, double *phi, double *y, double *X, double *Z, double *k, 
			   int *n, int *p, int *q, int *ni, double (*conanfun)(double x), 
			   double (*bfun)(double x), double (*cfun)(double x, double *phi), double *penalty, int *groupindex,
			   int *numgroup, int *penalize)
{
  struct data_glmme rod; 
  int nimax,sub,i,j,l,subin;
  nimax=max_int(*n,ni);
  
  double bit[NMAX][NMAX],kronbitZi[NMAX][NMAX],Jq[NMAX][NMAX],kronbitZiJq[NMAX][NMAX],
    Zi[NMAX][NMAX],*kronbitZiJqarray;

  kronbitZiJqarray = (double *) malloc(sum_int(*n,ni)**q*(*q+1)/2*sizeof(double));
  dupltmat(*q,Jq);

  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*q;j++)  {
      for (subin=0;subin<ni[sub];subin++)  {
 	Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
      }
      bit[0][j] = b[j+sub**q];
    }
    kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi);
    //bmp(ni[sub],*q**q,kronbitZi,"kronbitZi");
    matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
    //bmp(ni[sub],*q*(*q+1)/2,kronbitZiJq,"kronbitZiJq");
    //bmp(*q*(*q+1)/2,*q*(*q+1)/2,JqtkronbvariZitZiJq,"JqtkronbvariZitZiJq");
    for (j=0;j<*q*(*q+1)/2;j++)  {
      for (l=0;l<ni[sub];l++)  {
	kronbitZiJqarray[(sub==0)? l+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (l+ni[sub]*j)] = kronbitZiJq[l][j];
      }
    }
  }


  rod.theta = theta;
  rod.b = b;
  rod.phi = phi;
  rod.y = y;
  rod.X = X;
  rod.Z = Z;
  rod.k = k;
  rod.n = n;
  rod.p = p;
  rod.q = q;
  rod.ni = ni;
  rod.conanfun = conanfun;
  rod.bfun = bfun;
  rod.cfun = cfun;
  rod.penalty = penalty;
  rod.groupindex = groupindex;
  rod.numgroup = numgroup;
  rod.penalize = penalize;
  rod.kronbitZiJqarray=kronbitZiJqarray;



  int ninit=7,dometrop=1,err;
  double leftlimit=-5,rightlimit=5,thetajprev[1],thetajsamp[1],eta,etamat[sum_int(*n,ni)];

  for (j=0;j<*p+*q*(*q+1)/2;j++)  {
    // lets calculate etas to make it computationally faster
    for (sub=0;sub<*n;sub++)  {
      for (subin=0;subin<ni[sub];subin++)  {
	for (i=0,eta=0.0;i<*p;i++)
	  eta += X[(sub==0) ? subin+ni[sub]*i : sum_int(sub,ni)**p + (subin+ni[sub]*i)]*theta[i];
	for (i=0;i<*q*(*q+1)/2;i++)
	  eta += kronbitZiJqarray[(sub==0) ? subin+ni[sub]*i : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*i)]*theta[i+*p];
	if (j>=*p)  {
	  eta += -kronbitZiJqarray[(sub==0) ? subin+ni[sub]*(j-*p) : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*(j-*p))]*theta[j];
	}  else  {
	  eta += -X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)]*theta[j];
	}
	etamat[(sub==0) ? subin : sum_int(sub,ni) + subin] = eta;
      }
    }
    rod.j = &j;
    rod.etamat = etamat;
    //printf("rod.j=%d,j=%d\n",(rod.j)[0],j);
    thetajprev[0] = theta[j];
    //vp(*q,bsim[i],"bsim[i]");
    //vp(*q,rod.b,"rod.b");
    //glmme_lpd_theta_lasso(thetajprev[0],&rod);
    //printf("thetajprev=%f,glmme_lpd_re=%f\n",thetajprev[0],glmme_lpd_theta_lasso(thetajprev[0],&rod));
    err = arms_simple(ninit,&leftlimit,&rightlimit,glmme_lpd_theta_lasso,&rod,dometrop,thetajprev,thetajsamp);
    //printf("bjsamp=%f,glmme_lpd_re=%f\n",bjsamp[0],glmme_lpd_re(bjsamp[0],&rod));

    theta[j] = thetajsamp[0];
    rod.theta[j] = thetajsamp[0];
    //vp(*q,rod.b,"rod.b");
    //vp(*q,b,"b");
/*     if (err>0) { */
/*       printf("error code = %d\n",err); */
/*       exit(1); */
/*     } */
    //printf("Simulating\n");
  }
  free(kronbitZiJqarray);
}

// glmme_sim_theta_scad simulates one sample of theta of density of penalized Q function 
// of random effect theta under the scad penalty function
// To used when computing RE penalty estimate

void glmme_sim_theta_scad(double *theta, double *b, double *phi, double *y, double *X, double *Z, double *k, 
			   int *n, int *p, int *q, int *ni, double (*conanfun)(double x), 
			   double (*bfun)(double x), double (*cfun)(double x, double *phi), double *penalty, int *groupindex,
			   int *numgroup, int *penalize)
{
  struct data_glmme rod; 
  int nimax,sub,i,j,l,subin;
  nimax=max_int(*n,ni);
  
  double bit[NMAX][NMAX],kronbitZi[NMAX][NMAX],Jq[NMAX][NMAX],kronbitZiJq[NMAX][NMAX],
    Zi[NMAX][NMAX],*kronbitZiJqarray;

  kronbitZiJqarray = (double *) malloc(sum_int(*n,ni)**q*(*q+1)/2*sizeof(double));
  dupltmat(*q,Jq);
  for (sub=0;sub<*n;sub++)  {
    for (j=0;j<*q;j++)  {
      for (subin=0;subin<ni[sub];subin++)  {
 	Zi[subin][j] = Z[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**q + (subin+ni[sub]*j)];
      }
      bit[0][j] = b[j+sub**q];
    }
    kronecker(1,*q,ni[sub],*q,bit,Zi,kronbitZi);
    //bmp(ni[sub],*q**q,kronbitZi,"kronbitZi");
    matrix_mult(ni[sub],*q**q,*q*(*q+1)/2,kronbitZi,Jq,kronbitZiJq);
    //bmp(ni[sub],*q*(*q+1)/2,kronbitZiJq,"kronbitZiJq");
    //bmp(*q*(*q+1)/2,*q*(*q+1)/2,JqtkronbvariZitZiJq,"JqtkronbvariZitZiJq");
    for (j=0;j<*q*(*q+1)/2;j++)  {
      for (l=0;l<ni[sub];l++)  {
	kronbitZiJqarray[(sub==0)? l+ni[sub]*j : sum_int(sub,ni)**q*(*q+1)/2 + (l+ni[sub]*j)] = kronbitZiJq[l][j];
      }
    }
  }

  rod.theta = theta;
  rod.b = b;
  rod.phi = phi;
  rod.y = y;
  rod.X = X;
  rod.Z = Z;
  rod.k = k;
  rod.n = n;
  rod.p = p;
  rod.q = q;
  rod.ni = ni;
  rod.conanfun = conanfun;
  rod.bfun = bfun;
  rod.cfun = cfun;
  rod.penalty = penalty;
  rod.groupindex = groupindex;
  rod.numgroup = numgroup;
  rod.penalize = penalize;
  rod.kronbitZiJqarray=kronbitZiJqarray;

  int ninit=7,dometrop=1,err;
  double leftlimit=-5,rightlimit=5,thetajprev[1],thetajsamp[1],eta,etamat[sum_int(*n,ni)];

  for (j=0;j<*p+*q*(*q+1)/2;j++)  {
    // lets calculate etas to make it computationally faster
    for (sub=0;sub<*n;sub++)  {
      for (subin=0;subin<ni[sub];subin++)  {
	for (i=0,eta=0.0;i<*p;i++)
	  eta += X[(sub==0) ? subin+ni[sub]*i : sum_int(sub,ni)**p + (subin+ni[sub]*i)]*theta[i];
	for (i=0;i<*q*(*q+1)/2;i++)
	  eta += kronbitZiJqarray[(sub==0) ? subin+ni[sub]*i : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*i)]*theta[i+*p];
	if (j>=*p)  {
	  eta += -kronbitZiJqarray[(sub==0) ? subin+ni[sub]*(j-*p) : sum_int(sub,ni)**q*(*q+1)/2 + (subin+ni[sub]*(j-*p))]*theta[j];
	}  else  {
	  eta += -X[(sub==0) ? subin+ni[sub]*j : sum_int(sub,ni)**p + (subin+ni[sub]*j)]*theta[j];
	}
	etamat[(sub==0) ? subin : sum_int(sub,ni) + subin] = eta;
      }
    }
    rod.j = &j;
    rod.etamat = etamat;
    //printf("rod.j=%d,j=%d\n",(rod.j)[0],j);
    thetajprev[0] = theta[j];
    //vp(*q,bsim[i],"bsim[i]");
    //vp(*q,rod.b,"rod.b");
    //glmme_lpd_theta_scad(thetajprev[0],&rod);
    //printf("thetajprev=%f,glmme_lpd_re=%f\n",thetajprev[0],glmme_lpd_theta_scad(thetajprev[0],&rod));
    err = arms_simple(ninit,&leftlimit,&rightlimit,glmme_lpd_theta_scad,&rod,dometrop,thetajprev,thetajsamp);
    //printf("bjsamp=%f,glmme_lpd_re=%f\n",bjsamp[0],glmme_lpd_re(bjsamp[0],&rod));
    theta[j] = thetajsamp[0];
    rod.theta[j] = thetajsamp[0];
    //vp(*q,rod.b,"rod.b");
    //vp(*q,b,"b");
/*     if (err>0) { */
/*       printf("error code = %d\n",err); */
/*       exit(1); */
/*     } */
    //printf("Simulating\n");
  }
  free(kronbitZiJqarray);
}

// glmme_sim_lasso simulates nsim samples of random variable with density equal to penalized Q function of GLMM with lasso penalty function
// ARGUMENTS
// theta p+(q*(q+1)/2 vecotr equal to (beta,m) (if doing RE penalty estimation starting value otherwise current value of theta in EM alg)
// y, X, Z, k, n, p, q, ni as defined in lmeq.c
// simulation sample size
// conanfun, bfun, cfun as defined in lmeq.c
// fileb character filename of where to put sample of simulated random effects
// filetheta character filename of where to put sample of simulated theta's 
// penalty p+(q*(q+1)/2 vector of penalty values of corresponding element in theta
// groupindex p+(q*(q+1)/2 vector which indicates the group corresponding element in theta to which it belongs
// numgroup scalar number of groups
// penalize p+(q*(q+1)/2 vector which indicates whether element should be penalized, 1 yes else 0
// simtheta, 1 if simulating theta's else 0.....should be 1 if performing RE penalty estimation else 0

void glmme_sim_lasso(double *theta, double *phi, double *y, double *X, double *Z, double *k,
		     int *n, int *p, int *q, int *ni, int *nsim, double (*conanfun)(double x),
		     double (*bfun)(double x), double (*cfun)(double x, double *phi),  char **fileb,
		     char **filetheta, double *penalty, int *groupindex, int *numgroup, int *penalize, int *simtheta)
{
  FILE *fb,*ftheta;
  fb = fopen(*fileb,"w+"); // lets create new file
  ftheta = fopen(*filetheta,"w+"); // lets create new file
 
  double bsim[*n**q],thetasim[*p+*q*(*q+1)/2];
  setvectorto0(*n**q,bsim);

  //printf("n=%d,p=%d,q=%d,nsim=%d,simtheta=%d\n",*n,*p,*q,*nsim,*simtheta);

  if (simtheta[0])
    setvectorto0(*p+*q*(*q+1)/2,thetasim);
  else
    copyvec(*p+*q*(*q+1)/2,theta,thetasim);



  int j,l;

  double bsimmat[*nsim][*n**q],thetasimmat[*nsim][*p+*q*(*q+1)/2];

  for (l=0;l<*nsim;l++)  {
    // lets simulate bs, random effects
    glmme_sim_b(thetasim,bsim,phi,y,X,Z,k,n,p,q,ni,conanfun,bfun,cfun);
    //printf("I simulated the bs, ");
    //vp(*n**q,bsim,"bsim");
    if (*simtheta)  {  
	glmme_sim_theta_lasso(thetasim,bsim,phi,y,X,Z,k,n,p,q,ni,conanfun,bfun,cfun,penalty,groupindex,numgroup,penalize);
        //printf("I simulated the thetas, ");
    }
    copyvec(*n**q,bsim,bsimmat[l]);
    copyvec(*p+*q*(*q+1)/2,thetasim,thetasimmat[l]);
    //printf("simtehta=%d\n",simtheta[0]);
    //printf("At simulation %d\n",l);
  }
  for (l=0;l<*nsim;l++)  {
    for (j=0;j<*n**q;j++)  {
      fprintf(fb,"%11.5f",bsimmat[l][j]);
    }
    fprintf(fb,"\n");
  }
  fclose(fb);
  if (*simtheta)  {
    for (l=0;l<*nsim;l++)  {
      for (j=0;j<*p+*q*(*q+1)/2;j++)  {
	fprintf(ftheta,"%11.5f",thetasimmat[l][j]);
      }
      fprintf(ftheta,"\n");
    }
    fclose(ftheta);
  }
}

// glmme_sim_scad simulates nsim samples of random variable with density equal to penalized Q function of GLMM with scad penalty function
// ARGUMENTS
// theta p+(q*(q+1)/2 vecotr equal to (beta,m) (if doing RE penalty estimation starting value otherwise current value of theta in EM alg)
// y, X, Z, k, n, p, q, ni as defined in lmeq.c
// simulation sample size
// conanfun, bfun, cfun as defined in lmeq.c
// fileb character filename of where to put sample of simulated random effects
// filetheta character filename of where to put sample of simulated theta's 
// penalty p+(q*(q+1)/2 vector of penalty values of corresponding element in theta
// groupindex p+(q*(q+1)/2 vector which indicates the group corresponding element in theta to which it belongs
// numgroup scalar number of groups
// penalize p+(q*(q+1)/2 vector which indicates whether element should be penalized, 1 yes else 0
// simtheta, 1 if simulating theta's else 0.....should be 1 if performing RE penalty estimation else 0

void glmme_sim_scad(double *theta, double *phi, double *y, double *X, double *Z, double *k,
		     int *n, int *p, int *q, int *ni, int *nsim, double (*conanfun)(double x),
		     double (*bfun)(double x), double (*cfun)(double x, double *phi),  char **fileb,
		     char **filetheta, double *penalty, int *groupindex, int *numgroup, int *penalize, int *simtheta)
{
  FILE *fb,*ftheta;
  fb = fopen(*fileb,"w+"); // lets create new file
  ftheta = fopen(*filetheta,"w+"); // lets create new file
 
  double bsim[*n**q],thetasim[*p+*q*(*q+1)/2];
  setvectorto0(*n**q,bsim);
  if (*simtheta)
    setvectorto0(*p+*q*(*q+1)/2,thetasim);
  else
    copyvec(*p+*q*(*q+1)/2,theta,thetasim);

  int j,l;
  double bsimmat[*nsim][*n**q],thetasimmat[*nsim][*p+*q*(*q+1)/2];

  for (l=0;l<*nsim;l++)  {
    // lets simulate bs, random effects
    glmme_sim_b(thetasim,bsim,phi,y,X,Z,k,n,p,q,ni,conanfun,bfun,cfun);
    //vp(*n**q,bsim,"bsim");
    if (*simtheta)  
      glmme_sim_theta_scad(thetasim,bsim,phi,y,X,Z,k,n,p,q,ni,conanfun,bfun,cfun,penalty,groupindex,numgroup,penalize);
    copyvec(*n**q,bsim,bsimmat[l]);
    copyvec(*p+*q*(*q+1)/2,thetasim,thetasimmat[l]);
    //printf("simtehta=%d\n",simtheta[0]);
    //printf("At simulation %d\n",l);
  }
  for (l=0;l<*nsim;l++)  {
    for (j=0;j<*n**q;j++)  {
      fprintf(fb,"%11.5f",bsimmat[l][j]);
    }
    fprintf(fb,"\n");
  }
  fclose(fb);
  if (*simtheta)  {
    for (l=0;l<*nsim;l++)  {
      for (j=0;j<*p+*q*(*q+1)/2;j++)  {
	fprintf(ftheta,"%11.5f",thetasimmat[l][j]);
      }
      fprintf(ftheta,"\n");
    }
    fclose(ftheta);
  }
}

// version of glmm_sim_lasso for NLMM

void lmme_sim_lasso(double *theta, double *phi, double *y, double *X, double *Z, double *k,
		     int *n, int *p, int *q, int *ni, int *nsim, char **fileb,
		    char **filetheta, double *penalty, int *groupindex, int *numgroup, int *penalize, int *simtheta)  {
  glmme_sim_lasso(theta,phi,y,X,Z,k,n,p,q,ni,nsim,conanfun_lme,bfun_lme,cfun_lme,fileb,filetheta,penalty,groupindex,
		  numgroup,penalize,simtheta);
}

void lmme_sim_scad(double *theta, double *phi, double *y, double *X, double *Z, double *k,
		     int *n, int *p, int *q, int *ni, int *nsim, char **fileb,
		    char **filetheta, double *penalty, int *groupindex, int *numgroup, int *penalize, int *simtheta)  {
  glmme_sim_scad(theta,phi,y,X,Z,k,n,p,q,ni,nsim,conanfun_lme,bfun_lme,cfun_lme,fileb,filetheta,penalty,groupindex,
		 numgroup,penalize,simtheta);
}

/* void lmme_sim_theta_lasso(double *theta, double *b, double *phi, double *y, double *X, double *Z, double *k,  */
/* 			   int *n, int *p, int *q, int *ni, double *penalty, int *groupindex, */
/* 			   int *numgroup, int *penalize) */
/* { */
/*   struct data_glmme rod;  */
/*   int nimax,j; */
/*   nimax=max_int(*n,ni); */
  
/*   rod.theta = theta; */
/*   rod.phi = phi; */
/*   rod.y = y; */
/*   rod.X = X; */
/*   rod.Z = Z; */
/*   rod.k = k; */
/*   rod.n = n; */
/*   rod.p = p; */
/*   rod.q = q; */
/*   rod.ni = ni; */
/*   rod.penalty = penalty; */
/*   rod.groupindex = groupindex; */
/*   rod.numgroup = numgroup; */
/*   rod.penalize = penalize; */

/*   int ninit=7,dometrop=1,err; */
/*   double leftlimit=-5,rightlimit=5,thetajprev[1],thetajsamp[1]; */

/*   for (j=0;j<*p+*q*(*q+1)/2;j++)  { */
/*     // lets calculate etas to make it computationally faster */
/*     rod.j = &j; */
/*     //printf("rod.j=%d,j=%d\n",(rod.j)[0],j); */
/*     thetajprev[0] = theta[j]; */
/*     //vp(*q,bsim[i],"bsim[i]"); */
/*     //vp(*q,rod.b,"rod.b"); */
/*     //glmme_lpd_theta_lasso(thetajprev[0],&rod); */
/*     //printf("thetajprev=%f,glmme_lpd_re=%f\n",thetajprev[0],glmme_lpd_theta_lasso(thetajprev[0],&rod)); */
/*     err = arms_simple(ninit,&leftlimit,&rightlimit,lmme_lpd_theta_lasso,&rod,dometrop,thetajprev,thetajsamp); */
/*     //printf("bjsamp=%f,glmme_lpd_re=%f\n",bjsamp[0],glmme_lpd_re(bjsamp[0],&rod)); */
/*     theta[j] = thetajsamp[0]; */
/*     rod.theta[j] = thetajsamp[0]; */
/*     //vp(*q,rod.b,"rod.b"); */
/*     //vp(*q,b,"b"); */
/* /\*     if (err>0) { *\/ */
/* /\*       printf("error code = %d\n",err); *\/ */
/* /\*       exit(1); *\/ */
/* /\*     } *\/ */
/*     //printf("Simulating\n"); */
/*   } */
/* } */

/* /\* void lmme_sim_lasso(double *theta, double *phi, double *y, double *X, double *Z, double *k, *\/ */
/* /\* 		     int *n, int *p, int *q, int *ni, int *nsim, char **fileb, *\/ */
/* /\* 		     char **filetheta, double *penalty, int *groupindex, int *numgroup, int *penalize, int *simtheta) *\/ */
/* /\* { *\/ */
/* /\*   FILE *ftheta; *\/ */
/* /\*   ftheta = fopen(*filetheta,"w+"); // lets create new file *\/ */

/* /\*   double bsim[*n**q],thetasim[*p+*q*(*q+1)/2]; *\/ */
/* /\*   setvectorto0(*n**q,bsim); *\/ */

/* /\*   printf("n=%d,p=%d,q=%d,nsim=%d,simtheta=%d\n",*n,*p,*q,*nsim,*simtheta); *\/ */

/* /\*   int j,l; *\/ */

/* /\*   double thetasimmat[*nsim][*p+*q*(*q+1)/2]; *\/ */

/* /\*   for (l=0;l<*nsim;l++)  { *\/ */
/* /\*     // lets simulate bs, random effects *\/ */
/* /\*     //vp(*n**q,bsim,"bsim"); *\/ */
/* /\*     //return; *\/ */
/* /\*       if (*simtheta)  {   *\/ */
/* /\* 	lmme_sim_theta_lasso(thetasim,bsim,phi,y,X,Z,k,n,p,q,ni,penalty,groupindex,numgroup,penalize); *\/ */
/* /\*       } *\/ */
/* /\*     copyvec(*p+*q*(*q+1)/2,thetasim,thetasimmat[l]); *\/ */
/* /\*     //printf("simtehta=%d\n",simtheta[0]); *\/ */
/* /\*     printf("At simulation %d\n",l); *\/ */
/* /\*   } *\/ */
/* /\*   for (l=0;l<*nsim;l++)  { *\/ */
/* /\*     for (j=0;j<*p+*q*(*q+1)/2;j++)  { *\/ */
/* /\*       fprintf(ftheta,"%11.5f",thetasimmat[l][j]); *\/ */
/* /\*     } *\/ */
/* /\*     fprintf(ftheta,"\n"); *\/ */
/* /\*   } *\/ */
/* /\*   fclose(ftheta); *\/ */

/* /\* } *\/ */
