
library(MASS)


# normcomd computes the conditional distribution of normal mean vector of those ungiven by those given,
# mu and Sigma are the mean and covariance of the normal random variables
# given is a variable where the not oberved/given variables have NA while observed have values

normcond <- function(x,mu,Sigma)  {
  x <- as.matrix(x) ; mu <- as.matrix(mu)
  given <- !is.na(x)
  if (all(given)) return(list(mean=x,var=matrix(0,nrow=nrow(Sigma),ncol=ncol(Sigma))))
  if (nrow(x)==1) {
    if (is.na(x)) return(list(mean=mu,var=Sigma)) else return(list(mean=x,var=matrix(0,1,1)))
  }
  if (all(!given)) return(list(mean=mu,var=Sigma))
  mu1 <- mu[!given]
  mu2 <- mu[given]
  Sigma11 <- Sigma[!given,!given]
  Sigma12 <- matrix(Sigma[!given,given],ncol=sum(given))
  Sigma22 <- Sigma[given,given]
  mean.cond <- mu1+Sigma12%*%solve(Sigma22)%*%(x[given]-mu2)
  var.cond <- Sigma11-Sigma12%*%solve(Sigma22)%*%t(Sigma12)
  mu[!given] <- mean.cond
  mu[given] <- x[given]
  Sigma[!given,!given] <- var.cond
  Sigma[given,] <- Sigma[,given] <- 0
  out <- list(mean=mu,var=Sigma)
  return(out)
}

#missing data mechanism
missfun <- function(y,x,phi) {
  eta <- exp(-5+sum(-phi*x[-1])-2*y)
  out <- eta/(1+eta)
  out
}

#missing data mechanism 2
missfun2 <- function(y,x,phi) {
  alpha <- 1
  eta1 <- -2+phi*(y-sum(x[-c(1,2)]*alpha*muxx[-c(1,2)]))
  prob1 <- exp(eta1)/(1+exp(eta1))
  mis1 <- rbinom(1,1,prob1)
  eta2 <- eta1 - .1*mis1
  prob2 <- exp(eta2)/(1+exp(eta2))
  mis2 <- rbinom(1,1,prob2)
  out <- c(mis1,mis2)
}


### muxx is mean of covariates
muxx <- c(3,1.5,0,0,2,0,0,0)

### Sigmaxxcorr calulates corrrelation matrix of AR(1) process
Sigmaxxcorr <- function(rho,n)  {
  mat <- matrix(,nrow=n,ncol=n)
  for ( i in 1:n )  {
    for ( j in 1:n )  {
      mat[i,j] <- rho^(abs(i-j))
    }
  }
  return(mat)
}

funy <- function(phi)  {
n <- 1000 ; sigma <- 3
X <- mvrnorm(n,mu=rep(0,length=8),Sigma=Sigmaxxcorr(.5,8))
y <- X%*%muxx + sigma*rnorm(n)
miss <- do.call("rbind",lapply(1:n,function(x) missfun2(y[x],X[x,],phi)))
X[,1] <- ifelse(miss[,1],rep(NA,n),X[,1])
X[,2] <- ifelse(miss[,2],rep(NA,n),X[,2])
fun <- function(x)  {
  if ( is.na(x[1]) & !is.na(x[2]) ) out <- 1
  if ( !is.na(x[1]) & is.na(x[2]) ) out <- 2
  if ( is.na(x[1]) & is.na(x[2]) ) out <- 3
  if ( !is.na(x[1]) & !is.na(x[2]) ) out <- 4
  out
}
#debug(fun)
out <- table(apply(X,1,fun))/n
tmpout <- rep(NA,4)
names(tmpout) <- c(1,2,3,4)
tmpout[names(out)] <- out
tmpout
}

tabmissfun <- function(X)  {
fun <- function(x)  {
  if ( is.na(x[1]) & !is.na(x[2]) ) out <- 1
  if ( !is.na(x[1]) & is.na(x[2]) ) out <- 2
  if ( is.na(x[1]) & is.na(x[2]) ) out <- 3
  if ( !is.na(x[1]) & !is.na(x[2]) ) out <- 4
  out
}
out <- table(apply(X,1,fun))/nrow(X)
tmpout <- rep(0,4)
names(tmpout) <- c(1,2,3,4)
tmpout[names(out)] <- out
tmpout
}


